//--------------------------------------------------------------------
#ifndef _TIMBASIC_H_
#define _TIMBASIC_H_

//--------------------------------------------------------------------
//#ifdef _WINDOWS
//#include <windows.h>
//#endif
#include <iostream.h>
#include <math.h>

//--------------------------------------------------------------------
#ifndef TRUE
#define TRUE 1
#define FALSE 0
#endif

#ifndef M_PI
#define M_PI  (3.14159265359)
#define M_2PI (6.28318530718)
#endif
//--------------------------------------------------------------------
typedef double Float;
typedef int Int;
typedef bool Bool;

//--------------------------------------------------------------------
inline Int Sigma(const Int n)
{
    Int ret = 0;
    if(n>0) {
        ret = ((n*(n+1))/2);
    }
    return ret;
}

//--------------------------------------------------------------------
inline Int Faces(const Int n)
{
    return (Sigma(n-1) + Sigma(n-2));
}

//--------------------------------------------------------------------
class Int3
{
    public:
        Int3() : a_(0), b_(0), c_(0) {}
        Int3(Int a, Int b, Int c) : a_(a), b_(b), c_(c) {}
        ~Int3() {};

        Int &a() {return a_;}
        Int &b() {return b_;}
        Int &c() {return c_;}

        const Int &a() const {return a_;}
        const Int &b() const {return b_;}
        const Int &c() const {return c_;}

    private:
        Int a_;
        Int b_;
        Int c_;
};

//--------------------------------------------------------------------
inline ostream& operator<< (ostream &os, const Int3 &p)
{
    os << "(" << p.a() << "," << p.b() << "," << p.c() << ")";
    return os;
}

//--------------------------------------------------------------------
// scale_to_01 : linearly scale from [dmin, dmax] to [0,1]
//--------------------------------------------------------------------
inline Float scale_to_01 (const Float dmin, const Float dmax, const Float value)
{
    Float ret = dmax - dmin;
    if(fabs(ret) < .00001) { // TODO: GET A REAL EPSILON
        ret = 1.0;
    } else {
        ret = (value - dmin) / ret;
    }
    return ret;
}

//--------------------------------------------------------------------
// scale_from_01 : linearly scale from [0,1] to [dmin, dmax]
//--------------------------------------------------------------------
inline Float scale_from_01 (const Float dmin, const Float dmax, const Float value)
{
    Float ret = value * (dmax - dmin) + dmin;
    return ret;
}

//--------------------------------------------------------------------
// lin_scale : linearly scale from [dmin, dmax] to [imin, imax]
//--------------------------------------------------------------------
inline Float lin_scale (const Float dmin, const Float dmax,
    const Float imin, const Float imax, const Float value)
{
    Float ret;
    ret = scale_to_01(dmin, dmax, value);
    ret = scale_from_01(imin, imax, ret);
    return ret;
}

//--------------------------------------------------------------------
#endif // _TIMBASIC_H_
