//--------------------------------------------------------------------
#ifndef _PT_2_H_
#define _PT_2_H_

//--------------------------------------------------------------------
#include <basic.h>

//--------------------------------------------------------------------
class Pt2
{
    public:
        Pt2()
		{
			coord_[0] = 0.0;
			coord_[1] = 0.0;
		}

        Pt2(const Float &x, const Float &y)
		{
			coord_[0] = x;
			coord_[1] = y;
		}

        ~Pt2() {}

        Float &x() {return coord_[0];}
        Float &y() {return coord_[1];}

        const Float &x() const {return coord_[0];}
        const Float &y() const {return coord_[1];}

        Pt2& operator= (const Pt2 &p)
        {
			coord_[0] = p.coord_[0];
			coord_[1] = p.coord_[1];
			return *this;
		}

        Pt2 operator- () const
        {
			return Pt2(-coord_[0], -coord_[1]);
		}

        Pt2& operator+= (const Pt2 &p)
        {
			coord_[0] += p.coord_[0];
			coord_[1] += p.coord_[1];
			return *this;
		}

        Pt2& operator-= (const Pt2 &p)
        {
			coord_[0] -= p.coord_[0];
			coord_[1] -= p.coord_[1];
			return *this;
		}

        Pt2 operator+ (const Pt2 &p) const
        {
			return Pt2(coord_[0]+p.coord_[0],coord_[1]+p.coord_[1]);
		}

        Pt2 operator- (const Pt2 &p) const
        {
			return Pt2(coord_[0]-p.coord_[0],coord_[1]-p.coord_[1]);
		}

        Bool operator== (const Pt2 &p) const
        {
			return (coord_[0]==p.coord_[0]&&coord_[1]==p.coord_[1]);
		}

        Bool operator!= (const Pt2 &p) const
        {
			return (coord_[0]==p.coord_[0]||coord_[1]==p.coord_[1]);
		}

        Pt2 &operator*= (const Float &c)
        {
			coord_[0] *= c;
			coord_[1] *= c;
			return *this;
		}

        Pt2 &operator/= (const Float &c)
        { 
			coord_[0] /= c;
			coord_[1] /= c;
			return *this;
		}

        Float &operator[] (size_t idx)
        {
            return coord_[idx];
        }

        Float operator[] (size_t idx) const
        {
			return coord_[idx];
        }

        Float NormSquared() const
        {
            return (x()*x() + y()*y());
        }

        Float Norm() const
        {
            return sqrt(NormSquared());
        }

        Pt2 &Normalize() {
            Float n = Norm();
            if(n > 0) {
                (*this) /= n;
            }
			return *this;
        }

        Float NormalizeReturnNorm() {
            Float n = Norm();
            if(n > 0) {
                (*this) /= n;
            }
            return n;
        }


		Float *Coord() {return coord_;}

    private:
		Float coord_[2];
};

//--------------------------------------------------------------------
inline Pt2 operator* (const Pt2 &p, const Float &c)
{ 
	return Pt2(p.x() * c, p.y() * c);
}

//--------------------------------------------------------------------
inline Pt2 operator* (const Float &c, const Pt2 &p)
{ 
	return p * c; 
}

//--------------------------------------------------------------------
inline Pt2 operator/ (const Pt2 &p, const Float &c)
{ 
	return Pt2(p.x() / c, p.y() / c); 
}

//--------------------------------------------------------------------
inline Pt2 CrossProduct(const Pt2 &a, const Pt2 &b)
{
    Pt2 ret;
    ret.x() = a.x()*b.y() - a.y()*b.x();
    ret.y() = a.x()*b.y() - a.y()*b.x();
    return ret;
}

//--------------------------------------------------------------------
inline ostream& operator<< (ostream &os, const Pt2 &p)
{
    os << "(" << p.x() << "," << p.y() << ")";
    return os;
}

//--------------------------------------------------------------------
inline istream& operator>> (istream &is, Pt2 &p)
{
  is >> p.x() >> p.y();
  return is;
}

//--------------------------------------------------------------------
#endif // _PT_2_H_
