//--------------------------------------------------------------------
#ifndef _PT_3_H_
#define _PT_3_H_

//--------------------------------------------------------------------
#include<basic.h>

//--------------------------------------------------------------------
class Pt3
{
    public:
        Pt3()
		{
			coord_[0] = 0.0;
			coord_[1] = 0.0;
			coord_[2] = 0.0;
		}

        Pt3(const Float &x, const Float &y, const Float &z)
		{
			coord_[0] = x;
			coord_[1] = y;
			coord_[2] = z;
		}

        ~Pt3() {}

        Float &x() {return coord_[0];}
        Float &y() {return coord_[1];}
        Float &z() {return coord_[2];}

        const Float &x() const {return coord_[0];}
        const Float &y() const {return coord_[1];}
        const Float &z() const {return coord_[2];}

        Pt3& operator= (const Pt3 &p)
        {
			coord_[0] = p.coord_[0];
			coord_[1] = p.coord_[1];
			coord_[2] = p.coord_[2];
			return *this;
		}

        Pt3 operator- () const
        {
			return Pt3(-coord_[0], -coord_[1], -coord_[2]);
		}

        Pt3& operator+= (const Pt3 &p)
        {
			coord_[0] += p.coord_[0];
			coord_[1] += p.coord_[1];
			coord_[2] += p.coord_[2];
			return *this;
		}

        Pt3& operator-= (const Pt3 &p)
        {
			coord_[0] -= p.coord_[0];
			coord_[1] -= p.coord_[1];
			coord_[2] -= p.coord_[2];
			return *this;
		}

        Pt3 operator+ (const Pt3 &p) const
        {
			return Pt3(coord_[0]+p.coord_[0],coord_[1]+p.coord_[1],coord_[2]+p.coord_[2]);
		}

        Pt3 operator- (const Pt3 &p) const
        {
			return Pt3(coord_[0]-p.coord_[0],coord_[1]-p.coord_[1],coord_[2]-p.coord_[2]);
		}

        Bool operator== (const Pt3 &p) const
        {
			return (coord_[0]==p.coord_[0]&&coord_[1]==p.coord_[1]&&coord_[2]==p.coord_[2]);
		}

        Bool operator!= (const Pt3 &p) const
        {
			return (coord_[0]==p.coord_[0]||coord_[1]==p.coord_[1]||coord_[2]==p.coord_[2]);
		}

        Pt3 &operator*= (const Float &c)
        {
			coord_[0] *= c;
			coord_[1] *= c;
			coord_[2] *= c;
			return *this;
		}

        Pt3 &operator/= (const Float &c)
        { 
			coord_[0] /= c;
			coord_[1] /= c;
			coord_[2] /= c;
			return *this;
		}

        Float &operator[] (size_t idx)
        {
            return coord_[idx];
        }

        Float operator[] (size_t idx) const
        {
			return coord_[idx];
        }

        Float NormSquared() const
        {
            return (x()*x() + y()*y() + z()*z());
        }

        Float Norm() const
        {
            return sqrt(NormSquared());
        }

        Pt3 &Normalize() {
            Float n = Norm();
            if(n > 0) {
                (*this) /= n;
            }
			return *this;
        }

        Float NormalizeReturnNorm() {
            Float n = Norm();
            if(n > 0) {
                (*this) /= n;
            }
            return n;
        }


		Float *Coord() {return coord_;}

    private:
		Float coord_[3];
};

//--------------------------------------------------------------------
inline Pt3 operator* (const Pt3 &p, const Float &c)
{ 
	return Pt3(p.x() * c, p.y() * c, p.z() * c);
}

//--------------------------------------------------------------------
inline Pt3 operator* (const Float &c, const Pt3 &p)
{ 
	return p * c; 
}

//--------------------------------------------------------------------
inline Pt3 operator/ (const Pt3 &p, const Float &c)
{ 
	return Pt3(p.x() / c, p.y() / c, p.z() / c); 
}

//--------------------------------------------------------------------
inline Pt3 CrossProduct(const Pt3 &a, const Pt3 &b)
{
    Pt3 ret;
    ret.x() = a.y()*b.z() - a.z()*b.y();
    ret.y() = a.z()*b.x() - a.x()*b.z();
    ret.z() = a.x()*b.y() - a.y()*b.x();
    return ret;
}

//--------------------------------------------------------------------
inline ostream& operator<< (ostream &os, const Pt3 &p)
{
    os << "(" << p.x() << "," << p.y() << "," << p.z() << ")";
    return os;
}

//--------------------------------------------------------------------
inline istream& operator>> (istream &is, Pt3 &p)
{
  is >> p.x() >> p.y() >> p.z();
  return is;
}

//--------------------------------------------------------------------
#endif // _PT_3_H_
