//--------------------------------------------------------------------
#ifndef _PT_4_H_
#define _PT_4_H_

//--------------------------------------------------------------------
#include <basic.h>

//--------------------------------------------------------------------
class Pt4
{
    public:
        Pt4()
		{
			coord_[0] = 0.0;
			coord_[1] = 0.0;
			coord_[2] = 0.0;
			coord_[4] = 0.0;
		}

        Pt4(const Float &x, const Float &y, const Float &z, const Float &w)
		{
			coord_[0] = x;
			coord_[1] = y;
			coord_[2] = z;
			coord_[3] = z;
		}

        ~Pt4() {}

        Float &x() {return coord_[0];}
        Float &y() {return coord_[1];}
        Float &z() {return coord_[2];}
		Float &w() {return coord_[3];}

        const Float &x() const {return coord_[0];}
        const Float &y() const {return coord_[1];}
        const Float &z() const {return coord_[2];}
		const Float &w() const {return coord_[3];}

        Pt4& operator= (const Pt4 &p)
        {
			coord_[0] = p.coord_[0];
			coord_[1] = p.coord_[1];
			coord_[2] = p.coord_[2];
			coord_[3] = p.coord_[3];
			return *this;
		}

        Pt4 operator- () const
        {
			return Pt4(-coord_[0], -coord_[1], -coord_[2], -coord_[3]);
		}

        Pt4& operator+= (const Pt4 &p)
        {
			coord_[0] += p.coord_[0];
			coord_[1] += p.coord_[1];
			coord_[2] += p.coord_[2];
			coord_[3] += p.coord_[3];
			return *this;
		}

        Pt4& operator-= (const Pt4 &p)
        {
			coord_[0] -= p.coord_[0];
			coord_[1] -= p.coord_[1];
			coord_[2] -= p.coord_[2];
			coord_[3] -= p.coord_[3];
			return *this;
		}

        Pt4 operator+ (const Pt4 &p) const
        {
			return Pt4(coord_[0]+p.coord_[0],coord_[1]+p.coord_[1],coord_[2]+p.coord_[2],coord_[3]+p.coord_[3]);
		}

        Pt4 operator- (const Pt4 &p) const
        {
			return Pt4(coord_[0]-p.coord_[0],coord_[1]-p.coord_[1],coord_[2]-p.coord_[2],coord_[3]-p.coord_[3]);
		}

        Bool operator== (const Pt4 &p) const
        {
			return (coord_[0]==p.coord_[0]&&coord_[1]==p.coord_[1]&&coord_[2]==p.coord_[2]&&coord_[3]==p.coord_[3]);
		}

        Bool operator!= (const Pt4 &p) const
        {
			return (coord_[0]==p.coord_[0]||coord_[1]==p.coord_[1]||coord_[2]==p.coord_[2]||coord_[3]==p.coord_[3]);
		}

        Pt4 &operator*= (const Float &c)
        {
			coord_[0] *= c;
			coord_[1] *= c;
			coord_[2] *= c;
			coord_[3] *= c;
			return *this;
		}

        Pt4 &operator/= (const Float &c)
        { 
			coord_[0] /= c;
			coord_[1] /= c;
			coord_[2] /= c;
			coord_[3] /= c;
			return *this;
		}

        Float &operator[] (size_t idx)
        {
            return coord_[idx];
        }

        Float operator[] (size_t idx) const
        {
			return coord_[idx];
        }

        Float NormSquared() const
        {
            return (x()*x() + y()*y() + z()*z() + w()*w());
        }

        Float Norm() const
        {
            return sqrt(NormSquared());
        }

        Pt4 &Normalize() {
            Float n = Norm();
            if(n > 0) {
                (*this) /= n;
            }
			return *this;
        }

        Float NormalizeReturnNorm() {
            Float n = Norm();
            if(n > 0) {
                (*this) /= n;
            }
            return n;
        }


		Float *Coord() {return coord_;}

    private:
		Float coord_[4];
};

//--------------------------------------------------------------------
inline Pt4 operator* (const Pt4 &p, const Float &c)
{ 
	return Pt4(p.x() * c, p.y() * c, p.z() * c, p.w() * c);
}

//--------------------------------------------------------------------
inline Pt4 operator* (const Float &c, const Pt4 &p)
{ 
	return p * c; 
}

//--------------------------------------------------------------------
inline Pt4 operator/ (const Pt4 &p, const Float &c)
{ 
	return Pt4(p.x() / c, p.y() / c, p.z() / c, p.w() / c); 
}

//--------------------------------------------------------------------
/*
inline Pt4 CrossProduct(const Pt4 &a, const Pt4 &b)
{
    Pt4 ret;
    ret.x() = a.y()*b.z() - a.z()*b.y();
    ret.y() = a.z()*b.x() - a.x()*b.z();
    ret.z() = a.x()*b.y() - a.y()*b.x();
    return ret;
}
*/

//--------------------------------------------------------------------
inline ostream& operator<< (ostream &os, const Pt4 &p)
{
    os << "(" << p.x() << "," << p.y() << "," << p.z() << "," << p.w() << ")";
    return os;
}

//--------------------------------------------------------------------
inline istream& operator>> (istream &is, Pt4 &p) 
{
  is >> p.x() >> p.y() >> p.z() >> p.w();
  return is;
}

//--------------------------------------------------------------------
#endif // _PT_4_H_
