//--------------------------------------------------------------------
#include "stdafx.h"
#include <sphere.h>
#include <gl\gl.h>

//--------------------------------------------------------------------
Sphere::Sphere()
{
	setMajorAxis(Pt3(1.0, 0.0, 0.0));

	setNumXSteps(8);
	setNumYSteps(4);

	calc();
}

//--------------------------------------------------------------------
Sphere::~Sphere()
{

}

namespace
{
	void crossProduct(Pt3 &p1, Pt3 &p2, Pt3 &result)
	{
		result.x() = p1.y() * p2.z() - p1.z() * p2.y();
		result.y() = p1.z() * p2.x() - p1.x() * p2.z();
		result.z() = p1.x() * p2.y() - p1.y() * p2.x();
	}

	// assume result is 0,0,0 
	// http://astronomy.swin.edu.au/pbourke/geometry/rotate/
	// http://astronomy.swin.edu.au/pbourke/geometry/rotate/source1.c
	void rotateAboutVector(Pt3 &p1, Pt3 &rvec, double theta, Pt3 &result)
	{
		Pt3 normal(rvec);

		normal.Normalize();
	   double costheta,sintheta;

	   costheta = cos(theta);
	   sintheta = sin(theta);

	   double d1 = normal.x() * p1.x() + normal.y() * p1.y() + normal.z() * p1.z();

	   result.x() += (costheta + (1 - costheta) * normal.x() * normal.x()) * p1.x();
	   result.x() += ((1 - costheta) * normal.x() * normal.y() - normal.z() * sintheta) * p1.y();
	   result.x() += ((1 - costheta) * normal.x() * normal.z() + normal.y() * sintheta) * p1.z();

	   result.y() += ((1 - costheta) * normal.x() * normal.y() + normal.z() * sintheta) * p1.x();
	   result.y() += (costheta + (1 - costheta) * normal.y() * normal.y()) * p1.y();
	   result.y() += ((1 - costheta) * normal.y() * normal.z() - normal.x() * sintheta) * p1.z();

	   result.z() += ((1 - costheta) * normal.x() * normal.z() - normal.y() * sintheta) * p1.x();
	   result.z() += ((1 - costheta) * normal.y() * normal.z() + normal.x() * sintheta) * p1.y();
	   result.z() += (costheta + (1 - costheta) * normal.z() * normal.z()) * p1.z();

	   double d2 = normal.x() * result.x() + normal.y() * result.y() + normal.z() * result.z();
	}

	void testRot()
	{
		Pt3 p1(.7, .1, .9);

		Pt3 vec1(0,0,1);
		Pt3 corner3;

		rotateAboutVector(p1, vec1, 3.14159/6, corner3);
		Pt3 vec2(.707, 0, .707);
		Pt3 corner4;
		rotateAboutVector(p1, vec2, 3.14159/6, corner4);
		Pt3 vec3(0,.707, .707);
		Pt3 corner5;
		rotateAboutVector(p1, vec3, 3.14159/6, corner5);
		Pt3 vec4(.707, .707, 0);
		Pt3 corner6;
		rotateAboutVector(p1, vec4, 3.14159/6, corner6);
		Pt3 vec5(.59, .59, .59);
		Pt3 corner7;
		rotateAboutVector(p1, vec5, 3.14159/6, corner7);
		//  i = 0, x, y, z 0.556218, 0.436602, 0.9 d1 = 0.9, d2 = 0.9
// i = 1, x, y, z 0.678042, 0.0158919, 0.921958 d1 = 1.13137, d2 = 1.13137
// i = 2, x, y, z 0.88906, 0.401077, 0.598923 d1 = 0.707107, d2 = 0.707107
// i = 3, x, y, z 0.978005, -0.178005, 0.567291 d1 = 0.565685, d2 = 0.565685
// i = 4, x, y, z 0.913077, 0.104786, 0.682137 d1 = 0.981495, d2 = 0.981495

	}
	double vecLength(Pt3 &p1, Pt3 &p2)
	{
		Pt3 v(p2);
		v -= p1;

		double length = sqrt(v.x()*v.x() + v.y()*v.y() + v.z()*v.z());

		return length;
	}

	void drawLine(Pt3 &p1, Pt3 &p2)
	{
		glBegin(GL_LINES);

		glVertex3dv(p1.Coord());
		glVertex3dv(p2.Coord());
		glEnd();
	}

	void drawEdge( double x1, double y1, double z1, double x2, double y2, double z2)
	{
		static double stretch = 1.3;
		static double angle = .9;

		Pt3 p1(x1,y1,z1);
		Pt3 p2(x2,y2,z2);

		Pt3 mid(p1);
		mid += p2;
		mid *= 0.5;

		// drawLine(Pt3(), mid);

		double length = vecLength(mid, p2);

		Pt3 corner2(p2);
		corner2 -= mid;
//		corner2.Normalize();
//		Pt3 corner0(-corner2);

		corner2 *= stretch;
		corner2 += mid;

		Pt3 corner3;
		rotateAboutVector(corner2, mid, angle, corner3);

		Pt3 corner0(p1);
		corner0 -= mid;

		corner0 *= stretch;
		corner0 += mid;

		Pt3 corner1;
		rotateAboutVector(corner0, mid, angle, corner1);

		glBegin(GL_QUAD_STRIP);
		glVertex3dv(corner0.Coord());
		glVertex3dv(corner1.Coord());
		glVertex3dv(corner3.Coord());
		glVertex3dv(corner2.Coord());
		glEnd();
	}
}
//--------------------------------------------------------------------
void Sphere::Draw()
{
	//glColor3d(0.0, 1.0, 0.0);
	//testRot();

	material();


	double r1 = -0.04;
	double g1 = 9;
	double b1 = 1.04;
//Sphere Face #1
	r1 += 0.05;
	g1 = (g1>.08)?0.75:0.9;
	b1 -= 0.05;
	GLfloat localColor1[4]; localColor1[0] = r1; localColor1[1] = g1; localColor1[2] = b1; localColor1[3] = 1;
	glMaterialfv(GL_FRONT, GL_DIFFUSE, localColor1);
	drawEdge( 0.343279, 0, 0.939234, 0.106079, 0.326477, 0.939234);
	drawEdge( 0.343279, 0, 0.939234, 0.686557, 0, 0.727076);
	// drawEdge( 0.106079, 0.326477, 0.939234, 0.212158, 0.652955, 0.727076);
	drawEdge( 0.686557, 0, 0.727076, 0.792636, 0.326477, 0.514918);
	drawEdge( 0.212158, 0.652955, 0.727076, 0.555436, 0.652955, 0.514918);
	//- drawEdge( 0.792636, 0.326477, 0.514918, 0.555436, 0.652955, 0.514918);

	Pt3 pnt01(0.343279, 0, 0.939234);
	Pt3 pnt01a(0.555436, 0.652955, 0.514918);

	pnt01 += pnt01a;
	pnt01 *= 0.5;
	pnt01.Normalize();

	localColor1[1] -= 0.08;
	glMaterialfv(GL_FRONT, GL_DIFFUSE, localColor1);
	drawEdge(0.343279, 0, 0.939234, pnt01.x(), pnt01.y(), pnt01.z()); // p1, p6
	localColor1[1] -= 0.08;
	glMaterialfv(GL_FRONT, GL_DIFFUSE, localColor1);
	drawEdge(0.106079, 0.326477, 0.939234, pnt01.x(), pnt01.y(), pnt01.z()); // p2, p3
	localColor1[1] -= 0.08;
	glMaterialfv(GL_FRONT, GL_DIFFUSE, localColor1);
	drawEdge(0.212158, 0.652955, 0.727076, pnt01.x(), pnt01.y(), pnt01.z()); // p3, p4
	localColor1[1] -= 0.08;
	glMaterialfv(GL_FRONT, GL_DIFFUSE, localColor1);
	drawEdge(0.555436, 0.652955, 0.514918, pnt01.x(), pnt01.y(), pnt01.z()); // p4
	localColor1[1] -= 0.08;
	glMaterialfv(GL_FRONT, GL_DIFFUSE, localColor1);
	drawEdge(0.792636, 0.326477, 0.514918, pnt01.x(), pnt01.y(), pnt01.z()); // p5, p4
	localColor1[1] -= 0.08;
	glMaterialfv(GL_FRONT, GL_DIFFUSE, localColor1);
	drawEdge(0.686557, 0, 0.727076, pnt01.x(), pnt01.y(), pnt01.z()); // p6, p5


//Sphere Face #2
	r1 += 0.05;
	g1 = (g1>.08)?0.75:0.9;
	b1 -= 0.05;
	GLfloat localColor2[4]; localColor2[0] = r1; localColor2[1] = g1; localColor2[2] = b1; localColor2[3] = 1;
	glMaterialfv(GL_FRONT, GL_DIFFUSE, localColor2);
	drawEdge( 0.106079, 0.326477, 0.939234, -0.277718, 0.201774, 0.939234);
	drawEdge( 0.106079, 0.326477, 0.939234, 0.212158, 0.652955, 0.727076);
	// drawEdge( -0.277718, 0.201774, 0.939234, -0.555436, 0.403548, 0.727076);
	drawEdge( 0.212158, 0.652955, 0.727076, -0.0655604, 0.854729, 0.514918);
	drawEdge( -0.555436, 0.403548, 0.727076, -0.449358, 0.730026, 0.514918);
	//- drawEdge( -0.0655604, 0.854729, 0.514918, -0.449358, 0.730026, 0.514918);

	Pt3 pnt2(0.106079, 0.326477, 0.939234);
	Pt3 pnt2a(-0.449358, 0.730026, 0.514918);

	pnt2 += pnt2a;
	pnt2 *= 0.5;
	pnt2.Normalize();

	localColor2[1] -= 0.08;
	glMaterialfv(GL_FRONT, GL_DIFFUSE, localColor2);
	drawEdge(0.106079, 0.326477, 0.939234, pnt2.x(), pnt2.y(), pnt2.z()); // p1, p6
	localColor2[1] -= 0.08;
	glMaterialfv(GL_FRONT, GL_DIFFUSE, localColor2);
	drawEdge(-0.277718, 0.201774, 0.939234, pnt2.x(), pnt2.y(), pnt2.z()); // p2, p3
	localColor2[1] -= 0.08;
	glMaterialfv(GL_FRONT, GL_DIFFUSE, localColor2);
	drawEdge(-0.555436, 0.403548, 0.727076, pnt2.x(), pnt2.y(), pnt2.z()); // p3, p4
	localColor2[1] -= 0.08;
	glMaterialfv(GL_FRONT, GL_DIFFUSE, localColor2);
	drawEdge(-0.449358, 0.730026, 0.514918, pnt2.x(), pnt2.y(), pnt2.z()); // p4
	localColor2[1] -= 0.08;
	glMaterialfv(GL_FRONT, GL_DIFFUSE, localColor2);
	drawEdge(-0.0655604, 0.854729, 0.514918, pnt2.x(), pnt2.y(), pnt2.z()); // p5, p4
	localColor2[1] -= 0.08;
	glMaterialfv(GL_FRONT, GL_DIFFUSE, localColor1);
	drawEdge(0.212158, 0.652955, 0.727076, pnt2.x(), pnt2.y(), pnt2.z()); // p6, p5
	
	
//Sphere Face #3
	r1 += 0.05;
	g1 = (g1>.08)?0.75:0.9;
	b1 -= 0.05;
	GLfloat localColor3[4]; localColor3[0] = r1; localColor3[1] = g1; localColor3[2] = b1; localColor3[3] = 1;
	glMaterialfv(GL_FRONT, GL_DIFFUSE, localColor3);
	drawEdge( -0.277718, 0.201774, 0.939234, -0.277718, -0.201774, 0.939234);
	drawEdge( -0.277718, 0.201774, 0.939234, -0.555436, 0.403548, 0.727076);
	// drawEdge( -0.277718, -0.201774, 0.939234, -0.555436, -0.403548, 0.727076);
	drawEdge( -0.555436, 0.403548, 0.727076, -0.833155, 0.201774, 0.514918);
	drawEdge( -0.555436, -0.403548, 0.727076, -0.833155, -0.201774, 0.514918);
	//- drawEdge( -0.833155, 0.201774, 0.514918, -0.833155, -0.201774, 0.514918);

	Pt3 pnt3(-0.277718, 0.201774, 0.939234);
	Pt3 pnt3a( -0.833155, -0.201774, 0.514918);

	pnt3 += pnt3a;
	pnt3 *= 0.5;
	pnt3.Normalize();

	localColor3[1] -= 0.08;
	glMaterialfv(GL_FRONT, GL_DIFFUSE, localColor3);
	drawEdge(-0.277718, 0.201774, 0.939234, pnt3.x(), pnt3.y(), pnt3.z()); // p1, p6
	localColor3[1] -= 0.08;
	glMaterialfv(GL_FRONT, GL_DIFFUSE, localColor3);
	drawEdge( -0.277718, -0.201774, 0.939234, pnt3.x(), pnt3.y(), pnt3.z()); // p2, p3
	localColor3[1] -= 0.08;
	glMaterialfv(GL_FRONT, GL_DIFFUSE, localColor3);
	drawEdge(-0.555436, -0.403548, 0.727076, pnt3.x(), pnt3.y(), pnt3.z()); // p3, p4
	localColor3[1] -= 0.08;
	glMaterialfv(GL_FRONT, GL_DIFFUSE, localColor3);
	drawEdge( -0.833155, -0.201774, 0.514918, pnt3.x(), pnt3.y(), pnt3.z()); // p4
	localColor3[1] -= 0.08;
	glMaterialfv(GL_FRONT, GL_DIFFUSE, localColor3);
	drawEdge( -0.833155, 0.201774, 0.514918, pnt3.x(), pnt3.y(), pnt3.z()); // p5, p4
	localColor3[1] -= 0.08;
	glMaterialfv(GL_FRONT, GL_DIFFUSE, localColor3);
	drawEdge( -0.555436, 0.403548, 0.727076, pnt3.x(), pnt3.y(), pnt3.z()); // p6, p5

//Sphere Face #4
	r1 += 0.05;
	g1 = (g1>.08)?0.75:0.9;
	b1 -= 0.05;
	GLfloat localColor4[4]; localColor4[0] = r1; localColor4[1] = g1; localColor4[2] = b1; localColor4[3] = 1;
	glMaterialfv(GL_FRONT, GL_DIFFUSE, localColor4);
	drawEdge( -0.277718, -0.201774, 0.939234, 0.106079, -0.326477, 0.939234);
	drawEdge( -0.277718, -0.201774, 0.939234, -0.555436, -0.403548, 0.727076);
	// drawEdge( 0.106079, -0.326477, 0.939234, 0.212158, -0.652955, 0.727076);
	drawEdge( -0.555436, -0.403548, 0.727076, -0.449358, -0.730026, 0.514918);
	drawEdge( 0.212158, -0.652955, 0.727076, -0.0655604, -0.854729, 0.514918);
	//- drawEdge( -0.449358, -0.730026, 0.514918, -0.0655604, -0.854729, 0.514918);

	Pt3 pnt4(-0.277718, -0.201774, 0.939234);
	Pt3 pnt4a( -0.0655604, -0.854729, 0.514918);

	pnt4 += pnt4a;
	pnt4 *= 0.5;
	pnt4.Normalize();

	localColor4[1] -= 0.08;
	glMaterialfv(GL_FRONT, GL_DIFFUSE, localColor4);
	drawEdge(-0.277718, -0.201774, 0.939234, pnt4.x(), pnt4.y(), pnt4.z()); // p1, p6
	localColor4[1] -= 0.08;
	glMaterialfv(GL_FRONT, GL_DIFFUSE, localColor4);
	drawEdge(0.106079, -0.326477, 0.939234, pnt4.x(), pnt4.y(), pnt4.z()); // p2, p3
	localColor4[1] -= 0.08;
	glMaterialfv(GL_FRONT, GL_DIFFUSE, localColor4);
	drawEdge(0.212158, -0.652955, 0.727076, pnt4.x(), pnt4.y(), pnt4.z()); // p3, p4
	localColor4[1] -= 0.08;
	glMaterialfv(GL_FRONT, GL_DIFFUSE, localColor4);
	drawEdge( -0.0655604, -0.854729, 0.514918, pnt4.x(), pnt4.y(), pnt4.z()); // p4
	localColor4[1] -= 0.08;
	glMaterialfv(GL_FRONT, GL_DIFFUSE, localColor4);
	drawEdge(-0.449358, -0.730026, 0.514918, pnt4.x(), pnt4.y(), pnt4.z()); // p5, p4
	localColor4[1] -= 0.08;
	glMaterialfv(GL_FRONT, GL_DIFFUSE, localColor4);
	drawEdge(-0.555436, -0.403548, 0.727076, pnt4.x(), pnt4.y(), pnt4.z()); // p6, p5


//Sphere Face #5
	r1 += 0.05;
	g1 = (g1>.08)?0.75:0.9;
	b1 -= 0.05;
	GLfloat localColor5[4]; localColor5[0] = r1; localColor5[1] = g1; localColor5[2] = b1; localColor5[3] = 1;
	glMaterialfv(GL_FRONT, GL_DIFFUSE, localColor5);
	drawEdge( 0.106079, -0.326477, 0.939234, 0.343279, 1.13549e-15, 0.939234);
	drawEdge( 0.106079, -0.326477, 0.939234, 0.212158, -0.652955, 0.727076);
	// drawEdge( 0.343279, 1.13549e-15, 0.939234, 0.686557, 2.27099e-15, 0.727076);
	drawEdge( 0.212158, -0.652955, 0.727076, 0.555436, -0.652955, 0.514918);
	drawEdge( 0.686557, 2.27099e-15, 0.727076, 0.792636, -0.326477, 0.514918);
	//- drawEdge( 0.555436, -0.652955, 0.514918, 0.792636, -0.326477, 0.514918);

	Pt3 pnt5(0.106079, -0.326477, 0.939234);
	Pt3 pnt5a(0.792636, -0.326477, 0.514918);

	pnt5 += pnt5a;
	pnt5 *= 0.5;
	pnt5.Normalize();

	localColor5[1] -= 0.08;
	glMaterialfv(GL_FRONT, GL_DIFFUSE, localColor5);
	drawEdge(0.106079, -0.326477, 0.939234, pnt5.x(), pnt5.y(), pnt5.z()); // p1, p6
	localColor5[1] -= 0.08;
	glMaterialfv(GL_FRONT, GL_DIFFUSE, localColor5);
	drawEdge(0.343279, 1.13549e-15, 0.939234, pnt5.x(), pnt5.y(), pnt5.z()); // p2, p3
	localColor5[1] -= 0.08;
	glMaterialfv(GL_FRONT, GL_DIFFUSE, localColor5);
	drawEdge(0.686557, 2.27099e-15, 0.727076, pnt5.x(), pnt5.y(), pnt5.z()); // p3, p4
	localColor5[1] -= 0.08;
	glMaterialfv(GL_FRONT, GL_DIFFUSE, localColor5);
	drawEdge(0.792636, -0.326477, 0.514918, pnt5.x(), pnt5.y(), pnt5.z()); // p4
	localColor5[1] -= 0.08;
	glMaterialfv(GL_FRONT, GL_DIFFUSE, localColor5);
	drawEdge(0.555436, -0.652955, 0.514918, pnt5.x(), pnt5.y(), pnt5.z()); // p5, p4
	localColor5[1] -= 0.08;
	glMaterialfv(GL_FRONT, GL_DIFFUSE, localColor5);
	drawEdge(0.212158, -0.652955, 0.727076, pnt5.x(), pnt5.y(), pnt5.z()); // p6, p5


//Sphere Face #6
	r1 += 0.05;
	g1 = (g1>.08)?0.75:0.9;
	b1 -= 0.05;
	GLfloat localColor6[4]; localColor6[0] = r1; localColor6[1] = g1; localColor6[2] = b1; localColor6[3] = 1;
	glMaterialfv(GL_FRONT, GL_DIFFUSE, localColor6);
	drawEdge( 0.661515, 0.730026, -0.171639, 0.898715, 0.403548, -0.171639);
	drawEdge( 0.661515, 0.730026, -0.171639, 0.489876, 0.854729, 0.171639);
	// drawEdge( 0.898715, 0.403548, -0.171639, 0.964275, 0.201774, 0.171639);
	drawEdge( 0.489876, 0.854729, 0.171639, 0.555436, 0.652955, 0.514918);
	drawEdge( 0.964275, 0.201774, 0.171639, 0.792636, 0.326477, 0.514918);
	drawEdge( 0.555436, 0.652955, 0.514918, 0.792636, 0.326477, 0.514918);

	Pt3 pnt6(0.661515, 0.730026, -0.171639);
	Pt3 pnt6a(0.792636, 0.326477, 0.514918);

	pnt6 += pnt6a;
	pnt6 *= 0.5;
	pnt6.Normalize();

	localColor6[1] -= 0.08;
	glMaterialfv(GL_FRONT, GL_DIFFUSE, localColor6);
	drawEdge(0.661515, 0.730026, -0.171639, pnt6.x(), pnt6.y(), pnt6.z()); // p1, p6
	localColor6[1] -= 0.08;
	glMaterialfv(GL_FRONT, GL_DIFFUSE, localColor6);
	drawEdge(0.898715, 0.403548, -0.171639, pnt6.x(), pnt6.y(), pnt6.z()); // p2, p3
	localColor6[1] -= 0.08;
	glMaterialfv(GL_FRONT, GL_DIFFUSE, localColor6);
	drawEdge(0.964275, 0.201774, 0.171639, pnt6.x(), pnt6.y(), pnt6.z()); // p3, p4
	localColor6[1] -= 0.08;
	glMaterialfv(GL_FRONT, GL_DIFFUSE, localColor6);
	drawEdge(0.792636, 0.326477, 0.514918, pnt6.x(), pnt6.y(), pnt6.z()); // p4
	localColor6[1] -= 0.08;
	glMaterialfv(GL_FRONT, GL_DIFFUSE, localColor6);
	drawEdge(0.555436, 0.652955, 0.514918, pnt6.x(), pnt6.y(), pnt6.z()); // p5, p4
	localColor6[1] -= 0.08;
	glMaterialfv(GL_FRONT, GL_DIFFUSE, localColor6);
	drawEdge(0.489876, 0.854729, 0.171639, pnt6.x(), pnt6.y(), pnt6.z()); // p6, p5


//Sphere Face #7
	r1 += 0.05;
	g1 = (g1>.08)?0.75:0.9;
	b1 -= 0.05;
	GLfloat localColor7[4]; localColor7[0] = r1; localColor7[1] = g1; localColor7[2] = b1; localColor7[3] = 1;
	glMaterialfv(GL_FRONT, GL_DIFFUSE, localColor7);
	drawEdge( -0.489876, 0.854729, -0.171639, -0.106079, 0.979432, -0.171639);
	drawEdge( -0.489876, 0.854729, -0.171639, -0.661515, 0.730026, 0.171639);
	// drawEdge( -0.106079, 0.979432, -0.171639, 0.106079, 0.979432, 0.171639);
	drawEdge( -0.661515, 0.730026, 0.171639, -0.449358, 0.730026, 0.514918);
	drawEdge( 0.106079, 0.979432, 0.171639, -0.0655604, 0.854729, 0.514918);
	drawEdge( -0.449358, 0.730026, 0.514918, -0.0655604, 0.854729, 0.514918);

	Pt3 pnt7(-0.489876, 0.854729, -0.171639);
	Pt3 pnt7a(-0.0655604, 0.854729, 0.514918);

	pnt7 += pnt7a;
	pnt7 *= 0.5;
	pnt7.Normalize();

	localColor7[1] -= 0.08;
	glMaterialfv(GL_FRONT, GL_DIFFUSE, localColor7);
	drawEdge(-0.489876, 0.854729, -0.171639, pnt7.x(), pnt7.y(), pnt7.z()); // p1, p6
	localColor7[1] -= 0.08;
	glMaterialfv(GL_FRONT, GL_DIFFUSE, localColor7);
	drawEdge(-0.106079, 0.979432, -0.171639, pnt7.x(), pnt7.y(), pnt7.z()); // p2, p3
	localColor7[1] -= 0.08;
	glMaterialfv(GL_FRONT, GL_DIFFUSE, localColor7);
	drawEdge(0.106079, 0.979432, 0.171639, pnt7.x(), pnt7.y(), pnt7.z()); // p3, p4
	localColor7[1] -= 0.08;
	glMaterialfv(GL_FRONT, GL_DIFFUSE, localColor7);
	drawEdge(-0.0655604, 0.854729, 0.514918, pnt7.x(), pnt7.y(), pnt7.z()); // p4
	localColor7[1] -= 0.08;
	glMaterialfv(GL_FRONT, GL_DIFFUSE, localColor7);
	drawEdge(-0.449358, 0.730026, 0.514918, pnt7.x(), pnt7.y(), pnt7.z()); // p5, p4
	localColor7[1] -= 0.08;
	glMaterialfv(GL_FRONT, GL_DIFFUSE, localColor7);
	drawEdge(0 -0.661515, 0.730026, 0.171639, pnt7.x(), pnt7.y(), pnt7.z()); // p6, p5


//Sphere Face #8
	r1 += 0.05;
	g1 = (g1>.08)?0.75:0.9;
	b1 -= 0.05;
	GLfloat localColor8[4]; localColor8[0] = r1; localColor8[1] = g1; localColor8[2] = b1; localColor8[3] = 1;
	glMaterialfv(GL_FRONT, GL_DIFFUSE, localColor8);
	drawEdge( -0.964275, -0.201774, -0.171639, -0.964275, 0.201774, -0.171639);
	drawEdge( -0.964275, -0.201774, -0.171639, -0.898715, -0.403548, 0.171639);
	// drawEdge( -0.964275, 0.201774, -0.171639, -0.898715, 0.403548, 0.171639);
	drawEdge( -0.898715, -0.403548, 0.171639, -0.833155, -0.201774, 0.514918);
	drawEdge( -0.898715, 0.403548, 0.171639, -0.833155, 0.201774, 0.514918);
	drawEdge( -0.833155, -0.201774, 0.514918, -0.833155, 0.201774, 0.514918);

	Pt3 pnt8(-0.964275, -0.201774, -0.171639);
	Pt3 pnt8a(-0.833155, 0.201774, 0.514918);

	pnt8 += pnt8a;
	pnt8 *= 0.5;
	pnt8.Normalize();

	localColor8[1] -= 0.08;
	glMaterialfv(GL_FRONT, GL_DIFFUSE, localColor8);
	drawEdge(-0.964275, -0.201774, -0.171639, pnt8.x(), pnt8.y(), pnt8.z()); // p1, p6
	localColor8[1] -= 0.08;
	glMaterialfv(GL_FRONT, GL_DIFFUSE, localColor8);
	drawEdge(-0.964275, 0.201774, -0.171639, pnt8.x(), pnt8.y(), pnt8.z()); // p2, p3
	localColor8[1] -= 0.08;
	glMaterialfv(GL_FRONT, GL_DIFFUSE, localColor8);
	drawEdge(-0.898715, 0.403548, 0.171639, pnt8.x(), pnt8.y(), pnt8.z()); // p3, p4
	localColor8[1] -= 0.08;
	glMaterialfv(GL_FRONT, GL_DIFFUSE, localColor8);
	drawEdge(-0.833155, 0.201774, 0.514918, pnt8.x(), pnt8.y(), pnt8.z()); // p4
	localColor8[1] -= 0.08;
	glMaterialfv(GL_FRONT, GL_DIFFUSE, localColor8);
	drawEdge(-0.833155, -0.201774, 0.514918, pnt8.x(), pnt8.y(), pnt8.z()); // p5, p4
	localColor8[1] -= 0.08;
	glMaterialfv(GL_FRONT, GL_DIFFUSE, localColor8);
	drawEdge(-0.898715, -0.403548, 0.171639, pnt8.x(), pnt8.y(), pnt8.z()); // p6, p5


//Sphere Face #9
	r1 += 0.05;
	g1 = (g1>.08)?0.75:0.9;
	b1 -= 0.05;
	GLfloat localColor9[4]; localColor9[0] = r1; localColor9[1] = g1; localColor9[2] = b1; localColor9[3] = 1;
	glMaterialfv(GL_FRONT, GL_DIFFUSE, localColor9);
	drawEdge( -0.106079, -0.979432, -0.171639, -0.489876, -0.854729, -0.171639);
	drawEdge( -0.106079, -0.979432, -0.171639, 0.106079, -0.979432, 0.171639);
	// drawEdge( -0.489876, -0.854729, -0.171639, -0.661515, -0.730026, 0.171639);
	drawEdge( 0.106079, -0.979432, 0.171639, -0.0655604, -0.854729, 0.514918);
	drawEdge( -0.661515, -0.730026, 0.171639, -0.449358, -0.730026, 0.514918);
	drawEdge( -0.0655604, -0.854729, 0.514918, -0.449358, -0.730026, 0.514918);

	Pt3 pnt9(0.343279, 0, 0.939234);
	Pt3 pnt9a(0.555436, 0.652955, 0.514918);

	pnt9 += pnt9a;
	pnt9 *= 0.5;
	pnt9.Normalize();

	localColor9[1] -= 0.08;
	glMaterialfv(GL_FRONT, GL_DIFFUSE, localColor9);
	drawEdge(0.343279, 0, 0.939234, pnt9.x(), pnt9.y(), pnt9.z()); // p1, p6
	localColor9[1] -= 0.08;
	glMaterialfv(GL_FRONT, GL_DIFFUSE, localColor9);
	drawEdge(0.106079, 0.326477, 0.939234, pnt9.x(), pnt9.y(), pnt9.z()); // p2, p3
	localColor9[1] -= 0.08;
	glMaterialfv(GL_FRONT, GL_DIFFUSE, localColor9);
	drawEdge(0.212158, 0.652955, 0.727076, pnt9.x(), pnt9.y(), pnt9.z()); // p3, p4
	localColor9[1] -= 0.08;
	glMaterialfv(GL_FRONT, GL_DIFFUSE, localColor9);
	drawEdge(0.555436, 0.652955, 0.514918, pnt9.x(), pnt9.y(), pnt9.z()); // p4
	localColor9[1] -= 0.08;
	glMaterialfv(GL_FRONT, GL_DIFFUSE, localColor9);
	drawEdge(0.792636, 0.326477, 0.514918, pnt9.x(), pnt9.y(), pnt9.z()); // p5, p4
	localColor9[1] -= 0.08;
	glMaterialfv(GL_FRONT, GL_DIFFUSE, localColor9);
	drawEdge(0.686557, 0, 0.727076, pnt9.x(), pnt9.y(), pnt9.z()); // p6, p5


//Sphere Face #10
	r1 += 0.05;
	g1 = (g1>.08)?0.75:0.9;
	b1 -= 0.05;
	GLfloat localColor10[4]; localColor10[0] = r1; localColor10[1] = g1; localColor10[2] = b1; localColor10[3] = 1;
	glMaterialfv(GL_FRONT, GL_DIFFUSE, localColor10);
	drawEdge( 0.898715, -0.403548, -0.171639, 0.661515, -0.730026, -0.171639);
	drawEdge( 0.898715, -0.403548, -0.171639, 0.964275, -0.201774, 0.171639);
	// drawEdge( 0.661515, -0.730026, -0.171639, 0.489876, -0.854729, 0.171639);
	drawEdge( 0.964275, -0.201774, 0.171639, 0.792636, -0.326477, 0.514918);
	drawEdge( 0.489876, -0.854729, 0.171639, 0.555436, -0.652955, 0.514918);
	drawEdge( 0.792636, -0.326477, 0.514918, 0.555436, -0.652955, 0.514918);

	Pt3 pnt1770(0.343279, 0, 0.939234);
	Pt3 pnt1770a(0.555436, 0.652955, 0.514918);

	pnt1770 += pnt1770a;
	pnt1770 *= 0.5;
	pnt1770.Normalize();

	localColor10[1] -= 0.08;
	glMaterialfv(GL_FRONT, GL_DIFFUSE, localColor10);
	drawEdge(0.343279, 0, 0.939234, pnt1770.x(), pnt1770.y(), pnt1770.z()); // p1, p6
	localColor10[1] -= 0.08;
	glMaterialfv(GL_FRONT, GL_DIFFUSE, localColor10);
	drawEdge(0.106079, 0.326477, 0.939234, pnt1770.x(), pnt1770.y(), pnt1770.z()); // p2, p3
	localColor10[1] -= 0.08;
	glMaterialfv(GL_FRONT, GL_DIFFUSE, localColor10);
	drawEdge(0.212158, 0.652955, 0.727076, pnt1770.x(), pnt1770.y(), pnt1770.z()); // p3, p4
	localColor10[1] -= 0.08;
	glMaterialfv(GL_FRONT, GL_DIFFUSE, localColor10);
	drawEdge(0.555436, 0.652955, 0.514918, pnt1770.x(), pnt1770.y(), pnt1770.z()); // p4
	localColor10[1] -= 0.08;
	glMaterialfv(GL_FRONT, GL_DIFFUSE, localColor10);
	drawEdge(0.792636, 0.326477, 0.514918, pnt1770.x(), pnt1770.y(), pnt1770.z()); // p5, p4
	localColor10[1] -= 0.08;
	glMaterialfv(GL_FRONT, GL_DIFFUSE, localColor10);
	drawEdge(0.686557, 0, 0.727076, pnt1770.x(), pnt1770.y(), pnt1770.z()); // p6, p5


//Sphere Face #11
	r1 += 0.05;
	g1 = (g1>.08)?0.75:0.9;
	b1 -= 0.05;
	GLfloat localColor11[4]; localColor11[0] = r1; localColor11[1] = g1; localColor11[2] = b1; localColor11[3] = 1;
	glMaterialfv(GL_FRONT, GL_DIFFUSE, localColor11);
	drawEdge( 0.106079, 0.979432, 0.171639, 0.489876, 0.854729, 0.171639);
	drawEdge( 0.106079, 0.979432, 0.171639, -0.106079, 0.979432, -0.171639);
	// drawEdge( 0.489876, 0.854729, 0.171639, 0.661515, 0.730026, -0.171639);
	drawEdge( -0.106079, 0.979432, -0.171639, 0.0655604, 0.854729, -0.514918);
	drawEdge( 0.661515, 0.730026, -0.171639, 0.449358, 0.730026, -0.514918);
	drawEdge( 0.0655604, 0.854729, -0.514918, 0.449358, 0.730026, -0.514918);

	Pt3 pnt1771(0.343279, 0, 0.939234);
	Pt3 pnt1771a(0.555436, 0.652955, 0.514918);

	pnt1771 += pnt1771a;
	pnt1771 *= 0.5;
	pnt1771.Normalize();

	localColor11[1] -= 0.08;
	glMaterialfv(GL_FRONT, GL_DIFFUSE, localColor11);
	drawEdge(0.343279, 0, 0.939234, pnt1771.x(), pnt1771.y(), pnt1771.z()); // p1, p6
	localColor11[1] -= 0.08;
	glMaterialfv(GL_FRONT, GL_DIFFUSE, localColor11);
	drawEdge(0.106079, 0.326477, 0.939234, pnt1771.x(), pnt1771.y(), pnt1771.z()); // p2, p3
	localColor11[1] -= 0.08;
	glMaterialfv(GL_FRONT, GL_DIFFUSE, localColor11);
	drawEdge(0.212158, 0.652955, 0.727076, pnt1771.x(), pnt1771.y(), pnt1771.z()); // p3, p4
	localColor11[1] -= 0.08;
	glMaterialfv(GL_FRONT, GL_DIFFUSE, localColor11);
	drawEdge(0.555436, 0.652955, 0.514918, pnt1771.x(), pnt1771.y(), pnt1771.z()); // p4
	localColor11[1] -= 0.08;
	glMaterialfv(GL_FRONT, GL_DIFFUSE, localColor11);
	drawEdge(0.792636, 0.326477, 0.514918, pnt1771.x(), pnt1771.y(), pnt1771.z()); // p5, p4
	localColor11[1] -= 0.08;
	glMaterialfv(GL_FRONT, GL_DIFFUSE, localColor11);
	drawEdge(0.686557, 0, 0.727076, pnt1771.x(), pnt1771.y(), pnt1771.z()); // p6, p5


//Sphere Face #12
	r1 += 0.05;
	g1 = (g1>.08)?0.75:0.9;
	b1 -= 0.05;
	GLfloat localColor12[4]; localColor12[0] = r1; localColor12[1] = g1; localColor12[2] = b1; localColor12[3] = 1;
	glMaterialfv(GL_FRONT, GL_DIFFUSE, localColor12);
	drawEdge( -0.898715, 0.403548, 0.171639, -0.661515, 0.730026, 0.171639);
	drawEdge( -0.898715, 0.403548, 0.171639, -0.964275, 0.201774, -0.171639);
	// drawEdge( -0.661515, 0.730026, 0.171639, -0.489876, 0.854729, -0.171639);
	drawEdge( -0.964275, 0.201774, -0.171639, -0.792636, 0.326477, -0.514918);
	drawEdge( -0.489876, 0.854729, -0.171639, -0.555436, 0.652955, -0.514918);
	drawEdge( -0.792636, 0.326477, -0.514918, -0.555436, 0.652955, -0.514918);

	Pt3 pnt1772(0.343279, 0, 0.939234);
	Pt3 pnt1772a(0.555436, 0.652955, 0.514918);

	pnt1772 += pnt1772a;
	pnt1772 *= 0.5;
	pnt1772.Normalize();

	localColor12[1] -= 0.08;
	glMaterialfv(GL_FRONT, GL_DIFFUSE, localColor12);
	drawEdge(0.343279, 0, 0.939234, pnt1772.x(), pnt1772.y(), pnt1772.z()); // p1, p6
	localColor12[1] -= 0.08;
	glMaterialfv(GL_FRONT, GL_DIFFUSE, localColor12);
	drawEdge(0.106079, 0.326477, 0.939234, pnt1772.x(), pnt1772.y(), pnt1772.z()); // p2, p3
	localColor12[1] -= 0.08;
	glMaterialfv(GL_FRONT, GL_DIFFUSE, localColor12);
	drawEdge(0.212158, 0.652955, 0.727076, pnt1772.x(), pnt1772.y(), pnt1772.z()); // p3, p4
	localColor12[1] -= 0.08;
	glMaterialfv(GL_FRONT, GL_DIFFUSE, localColor12);
	drawEdge(0.555436, 0.652955, 0.514918, pnt1772.x(), pnt1772.y(), pnt1772.z()); // p4
	localColor12[1] -= 0.08;
	glMaterialfv(GL_FRONT, GL_DIFFUSE, localColor12);
	drawEdge(0.792636, 0.326477, 0.514918, pnt1772.x(), pnt1772.y(), pnt1772.z()); // p5, p4
	localColor12[1] -= 0.08;
	glMaterialfv(GL_FRONT, GL_DIFFUSE, localColor12);
	drawEdge(0.686557, 0, 0.727076, pnt1772.x(), pnt1772.y(), pnt1772.z()); // p6, p5


//Sphere Face #13
	r1 += 0.05;
	g1 = (g1>.08)?0.75:0.9;
	b1 -= 0.05;
	GLfloat localColor13[4]; localColor13[0] = r1; localColor13[1] = g1; localColor13[2] = b1; localColor13[3] = 1;
	glMaterialfv(GL_FRONT, GL_DIFFUSE, localColor13);
	drawEdge( -0.661515, -0.730026, 0.171639, -0.898715, -0.403548, 0.171639);
	drawEdge( -0.661515, -0.730026, 0.171639, -0.489876, -0.854729, -0.171639);
	// drawEdge( -0.898715, -0.403548, 0.171639, -0.964275, -0.201774, -0.171639);
	drawEdge( -0.489876, -0.854729, -0.171639, -0.555436, -0.652955, -0.514918);
	drawEdge( -0.964275, -0.201774, -0.171639, -0.792636, -0.326477, -0.514918);
	drawEdge( -0.555436, -0.652955, -0.514918, -0.792636, -0.326477, -0.514918);

	Pt3 pnt1773(0.343279, 0, 0.939234);
	Pt3 pnt1773a(0.555436, 0.652955, 0.514918);

	pnt1773 += pnt1773a;
	pnt1773 *= 0.5;
	pnt1773.Normalize();

	localColor13[1] -= 0.08;
	glMaterialfv(GL_FRONT, GL_DIFFUSE, localColor13);
	drawEdge(0.343279, 0, 0.939234, pnt1773.x(), pnt1773.y(), pnt1773.z()); // p1, p6
	localColor13[1] -= 0.08;
	glMaterialfv(GL_FRONT, GL_DIFFUSE, localColor13);
	drawEdge(0.106079, 0.326477, 0.939234, pnt1773.x(), pnt1773.y(), pnt1773.z()); // p2, p3
	localColor13[1] -= 0.08;
	glMaterialfv(GL_FRONT, GL_DIFFUSE, localColor13);
	drawEdge(0.212158, 0.652955, 0.727076, pnt1773.x(), pnt1773.y(), pnt1773.z()); // p3, p4
	localColor13[1] -= 0.08;
	glMaterialfv(GL_FRONT, GL_DIFFUSE, localColor13);
	drawEdge(0.555436, 0.652955, 0.514918, pnt1773.x(), pnt1773.y(), pnt1773.z()); // p4
	localColor13[1] -= 0.08;
	glMaterialfv(GL_FRONT, GL_DIFFUSE, localColor13);
	drawEdge(0.792636, 0.326477, 0.514918, pnt1773.x(), pnt1773.y(), pnt1773.z()); // p5, p4
	localColor13[1] -= 0.08;
	glMaterialfv(GL_FRONT, GL_DIFFUSE, localColor13);
	drawEdge(0.686557, 0, 0.727076, pnt1773.x(), pnt1773.y(), pnt1773.z()); // p6, p5


//Sphere Face #14
	r1 += 0.05;
	g1 = (g1>.08)?0.75:0.9;
	b1 -= 0.05;
	GLfloat localColor14[4]; localColor14[0] = r1; localColor14[1] = g1; localColor14[2] = b1; localColor14[3] = 1;
	glMaterialfv(GL_FRONT, GL_DIFFUSE, localColor14);
	drawEdge( 0.489876, -0.854729, 0.171639, 0.106079, -0.979432, 0.171639);
	drawEdge( 0.489876, -0.854729, 0.171639, 0.661515, -0.730026, -0.171639);
	// drawEdge( 0.106079, -0.979432, 0.171639, -0.106079, -0.979432, -0.171639);
	drawEdge( 0.661515, -0.730026, -0.171639, 0.449358, -0.730026, -0.514918);
	drawEdge( -0.106079, -0.979432, -0.171639, 0.0655604, -0.854729, -0.514918);
	drawEdge( 0.449358, -0.730026, -0.514918, 0.0655604, -0.854729, -0.514918);

	Pt3 pnt1774(0.343279, 0, 0.939234);
	Pt3 pnt1774a(0.555436, 0.652955, 0.514918);

	pnt1774 += pnt1774a;
	pnt1774 *= 0.5;
	pnt1774.Normalize();

	localColor14[1] -= 0.08;
	glMaterialfv(GL_FRONT, GL_DIFFUSE, localColor14);
	drawEdge(0.343279, 0, 0.939234, pnt1774.x(), pnt1774.y(), pnt1774.z()); // p1, p6
	localColor14[1] -= 0.08;
	glMaterialfv(GL_FRONT, GL_DIFFUSE, localColor14);
	drawEdge(0.106079, 0.326477, 0.939234, pnt1774.x(), pnt1774.y(), pnt1774.z()); // p2, p3
	localColor14[1] -= 0.08;
	glMaterialfv(GL_FRONT, GL_DIFFUSE, localColor14);
	drawEdge(0.212158, 0.652955, 0.727076, pnt1774.x(), pnt1774.y(), pnt1774.z()); // p3, p4
	localColor14[1] -= 0.08;
	glMaterialfv(GL_FRONT, GL_DIFFUSE, localColor14);
	drawEdge(0.555436, 0.652955, 0.514918, pnt1774.x(), pnt1774.y(), pnt1774.z()); // p4
	localColor14[1] -= 0.08;
	glMaterialfv(GL_FRONT, GL_DIFFUSE, localColor14);
	drawEdge(0.792636, 0.326477, 0.514918, pnt1774.x(), pnt1774.y(), pnt1774.z()); // p5, p4
	localColor14[1] -= 0.08;
	glMaterialfv(GL_FRONT, GL_DIFFUSE, localColor14);
	drawEdge(0.686557, 0, 0.727076, pnt1774.x(), pnt1774.y(), pnt1774.z()); // p6, p5


//Sphere Face #15
	r1 += 0.05;
	g1 = (g1>.08)?0.75:0.9;
	b1 -= 0.05;
	GLfloat localColor15[4]; localColor15[0] = r1; localColor15[1] = g1; localColor15[2] = b1; localColor15[3] = 1;
	glMaterialfv(GL_FRONT, GL_DIFFUSE, localColor15);
	drawEdge( 0.964275, 0.201774, 0.171639, 0.964275, -0.201774, 0.171639);
	drawEdge( 0.964275, 0.201774, 0.171639, 0.898715, 0.403548, -0.171639);
	// drawEdge( 0.964275, -0.201774, 0.171639, 0.898715, -0.403548, -0.171639);
	drawEdge( 0.898715, 0.403548, -0.171639, 0.833155, 0.201774, -0.514918);
	drawEdge( 0.898715, -0.403548, -0.171639, 0.833155, -0.201774, -0.514918);
	drawEdge( 0.833155, 0.201774, -0.514918, 0.833155, -0.201774, -0.514918);

	Pt3 pnt1775(0.343279, 0, 0.939234);
	Pt3 pnt1775a(0.555436, 0.652955, 0.514918);

	pnt1775 += pnt1775a;
	pnt1775 *= 0.5;
	pnt1775.Normalize();

	localColor15[1] -= 0.08;
	glMaterialfv(GL_FRONT, GL_DIFFUSE, localColor15);
	drawEdge(0.343279, 0, 0.939234, pnt1775.x(), pnt1775.y(), pnt1775.z()); // p1, p6
	localColor15[1] -= 0.08;
	glMaterialfv(GL_FRONT, GL_DIFFUSE, localColor15);
	drawEdge(0.106079, 0.326477, 0.939234, pnt1775.x(), pnt1775.y(), pnt1775.z()); // p2, p3
	localColor15[1] -= 0.08;
	glMaterialfv(GL_FRONT, GL_DIFFUSE, localColor15);
	drawEdge(0.212158, 0.652955, 0.727076, pnt1775.x(), pnt1775.y(), pnt1775.z()); // p3, p4
	localColor15[1] -= 0.08;
	glMaterialfv(GL_FRONT, GL_DIFFUSE, localColor15);
	drawEdge(0.555436, 0.652955, 0.514918, pnt1775.x(), pnt1775.y(), pnt1775.z()); // p4
	localColor15[1] -= 0.08;
	glMaterialfv(GL_FRONT, GL_DIFFUSE, localColor15);
	drawEdge(0.792636, 0.326477, 0.514918, pnt1775.x(), pnt1775.y(), pnt1775.z()); // p5, p4
	localColor15[1] -= 0.08;
	glMaterialfv(GL_FRONT, GL_DIFFUSE, localColor15);
	drawEdge(0.686557, 0, 0.727076, pnt1775.x(), pnt1775.y(), pnt1775.z()); // p6, p5


//Sphere Face #16
	r1 += 0.05;
	g1 = (g1>.08)?0.75:0.9;
	b1 -= 0.05;
	GLfloat localColor16[4]; localColor16[0] = r1; localColor16[1] = g1; localColor16[2] = b1; localColor16[3] = 1;
	glMaterialfv(GL_FRONT, GL_DIFFUSE, localColor16);
	drawEdge( 0.277718, 0.201774, -0.939234, -0.106079, 0.326477, -0.939234);
	drawEdge( 0.277718, 0.201774, -0.939234, 0.555436, 0.403548, -0.727076);
	// drawEdge( -0.106079, 0.326477, -0.939234, -0.212158, 0.652955, -0.727076);
	drawEdge( 0.555436, 0.403548, -0.727076, 0.449358, 0.730026, -0.514918);
	drawEdge( -0.212158, 0.652955, -0.727076, 0.0655604, 0.854729, -0.514918);
	//- drawEdge( 0.449358, 0.730026, -0.514918, 0.0655604, 0.854729, -0.514918);

	Pt3 pnt1776(0.343279, 0, 0.939234);
	Pt3 pnt1776a(0.555436, 0.652955, 0.514918);

	pnt1776 += pnt1776a;
	pnt1776 *= 0.5;
	pnt1776.Normalize();

	localColor16[1] -= 0.08;
	glMaterialfv(GL_FRONT, GL_DIFFUSE, localColor16);
	drawEdge(0.343279, 0, 0.939234, pnt1776.x(), pnt1776.y(), pnt1776.z()); // p1, p6
	localColor16[1] -= 0.08;
	glMaterialfv(GL_FRONT, GL_DIFFUSE, localColor16);
	drawEdge(0.106079, 0.326477, 0.939234, pnt1776.x(), pnt1776.y(), pnt1776.z()); // p2, p3
	localColor16[1] -= 0.08;
	glMaterialfv(GL_FRONT, GL_DIFFUSE, localColor16);
	drawEdge(0.212158, 0.652955, 0.727076, pnt1776.x(), pnt1776.y(), pnt1776.z()); // p3, p4
	localColor16[1] -= 0.08;
	glMaterialfv(GL_FRONT, GL_DIFFUSE, localColor16);
	drawEdge(0.555436, 0.652955, 0.514918, pnt1776.x(), pnt1776.y(), pnt1776.z()); // p4
	localColor16[1] -= 0.08;
	glMaterialfv(GL_FRONT, GL_DIFFUSE, localColor16);
	drawEdge(0.792636, 0.326477, 0.514918, pnt1776.x(), pnt1776.y(), pnt1776.z()); // p5, p4
	localColor16[1] -= 0.08;
	glMaterialfv(GL_FRONT, GL_DIFFUSE, localColor16);
	drawEdge(0.686557, 0, 0.727076, pnt1776.x(), pnt1776.y(), pnt1776.z()); // p6, p5


//Sphere Face #17
	r1 += 0.05;
	g1 = (g1>.08)?0.75:0.9;
	b1 -= 0.05;
	GLfloat localColor17[4]; localColor17[0] = r1; localColor17[1] = g1; localColor17[2] = b1; localColor17[3] = 1;
	glMaterialfv(GL_FRONT, GL_DIFFUSE, localColor17);
	drawEdge( -0.106079, 0.326477, -0.939234, -0.343279, 8.7264e-16, -0.939234);
	drawEdge( -0.106079, 0.326477, -0.939234, -0.212158, 0.652955, -0.727076);
	// drawEdge( -0.343279, 8.7264e-16, -0.939234, -0.686557, 1.74528e-15, -0.727076);
	drawEdge( -0.212158, 0.652955, -0.727076, -0.555436, 0.652955, -0.514918);
	drawEdge( -0.686557, 1.74528e-15, -0.727076, -0.792636, 0.326477, -0.514918);
	//- drawEdge( -0.555436, 0.652955, -0.514918, -0.792636, 0.326477, -0.514918);

	Pt3 pnt177(0.343279, 0, 0.939234);
	Pt3 pnt177a(0.555436, 0.652955, 0.514918);

	pnt177 += pnt177a;
	pnt177 *= 0.5;
	pnt177.Normalize();

	localColor17[1] -= 0.08;
	glMaterialfv(GL_FRONT, GL_DIFFUSE, localColor17);
	drawEdge(0.343279, 0, 0.939234, pnt177.x(), pnt177.y(), pnt177.z()); // p1, p6
	localColor17[1] -= 0.08;
	glMaterialfv(GL_FRONT, GL_DIFFUSE, localColor17);
	drawEdge(0.106079, 0.326477, 0.939234, pnt177.x(), pnt177.y(), pnt177.z()); // p2, p3
	localColor17[1] -= 0.08;
	glMaterialfv(GL_FRONT, GL_DIFFUSE, localColor17);
	drawEdge(0.212158, 0.652955, 0.727076, pnt177.x(), pnt177.y(), pnt177.z()); // p3, p4
	localColor17[1] -= 0.08;
	glMaterialfv(GL_FRONT, GL_DIFFUSE, localColor17);
	drawEdge(0.555436, 0.652955, 0.514918, pnt177.x(), pnt177.y(), pnt177.z()); // p4
	localColor17[1] -= 0.08;
	glMaterialfv(GL_FRONT, GL_DIFFUSE, localColor17);
	drawEdge(0.792636, 0.326477, 0.514918, pnt177.x(), pnt177.y(), pnt177.z()); // p5, p4
	localColor17[1] -= 0.08;
	glMaterialfv(GL_FRONT, GL_DIFFUSE, localColor17);
	drawEdge(0.686557, 0, 0.727076, pnt177.x(), pnt177.y(), pnt177.z()); // p6, p5


//Sphere Face #18
	r1 += 0.05;
	g1 = (g1>.08)?0.75:0.9;
	b1 -= 0.05;
	GLfloat localColor18[4]; localColor18[0] = r1; localColor18[1] = g1; localColor18[2] = b1; localColor18[3] = 1;
	glMaterialfv(GL_FRONT, GL_DIFFUSE, localColor18);
	drawEdge( -0.343279, 2.62855e-16, -0.939234, -0.106079, -0.326477, -0.939234);
	drawEdge( -0.343279, 2.62855e-16, -0.939234, -0.686557, 5.25709e-16, -0.727076);
	// drawEdge( -0.106079, -0.326477, -0.939234, -0.212158, -0.652955, -0.727076);
	drawEdge( -0.686557, 5.25709e-16, -0.727076, -0.792636, -0.326477, -0.514918);
	drawEdge( -0.212158, -0.652955, -0.727076, -0.555436, -0.652955, -0.514918);
	//- drawEdge( -0.792636, -0.326477, -0.514918, -0.555436, -0.652955, -0.514918);

	Pt3 pnt18(0.343279, 0, 0.939234);
	Pt3 pnt18a(0.555436, 0.652955, 0.514918);

	pnt18 += pnt18a;
	pnt18 *= 0.5;
	pnt18.Normalize();

	localColor18[1] -= 0.08;
	glMaterialfv(GL_FRONT, GL_DIFFUSE, localColor18);
	drawEdge(0.343279, 0, 0.939234, pnt18.x(), pnt18.y(), pnt18.z()); // p1, p6
	localColor18[1] -= 0.08;
	glMaterialfv(GL_FRONT, GL_DIFFUSE, localColor18);
	drawEdge(0.106079, 0.326477, 0.939234, pnt18.x(), pnt18.y(), pnt18.z()); // p2, p3
	localColor18[1] -= 0.08;
	glMaterialfv(GL_FRONT, GL_DIFFUSE, localColor18);
	drawEdge(0.212158, 0.652955, 0.727076, pnt18.x(), pnt18.y(), pnt18.z()); // p3, p4
	localColor18[1] -= 0.08;
	glMaterialfv(GL_FRONT, GL_DIFFUSE, localColor18);
	drawEdge(0.555436, 0.652955, 0.514918, pnt18.x(), pnt18.y(), pnt18.z()); // p4
	localColor18[1] -= 0.08;
	glMaterialfv(GL_FRONT, GL_DIFFUSE, localColor18);
	drawEdge(0.792636, 0.326477, 0.514918, pnt18.x(), pnt18.y(), pnt18.z()); // p5, p4
	localColor18[1] -= 0.08;
	glMaterialfv(GL_FRONT, GL_DIFFUSE, localColor18);
	drawEdge(0.686557, 0, 0.727076, pnt18.x(), pnt18.y(), pnt18.z()); // p6, p5


//Sphere Face #19
	r1 += 0.05;
	g1 = (g1>.08)?0.75:0.9;
	b1 -= 0.05;
	GLfloat localColor19[4]; localColor19[0] = r1; localColor19[1] = g1; localColor19[2] = b1; localColor19[3] = 1;
	glMaterialfv(GL_FRONT, GL_DIFFUSE, localColor19);
	drawEdge( -0.106079, -0.326477, -0.939234, 0.277718, -0.201774, -0.939234);
	drawEdge( -0.106079, -0.326477, -0.939234, -0.212158, -0.652955, -0.727076);
	// drawEdge( 0.277718, -0.201774, -0.939234, 0.555436, -0.403548, -0.727076);
	drawEdge( -0.212158, -0.652955, -0.727076, 0.0655604, -0.854729, -0.514918);
	drawEdge( 0.555436, -0.403548, -0.727076, 0.449358, -0.730026, -0.514918);
	//- drawEdge( 0.0655604, -0.854729, -0.514918, 0.449358, -0.730026, -0.514918);

	Pt3 pnt19(0.343279, 0, 0.939234);
	Pt3 pnt19a(0.555436, 0.652955, 0.514918);

	pnt19 += pnt19a;
	pnt19 *= 0.5;
	pnt19.Normalize();

	localColor19[1] -= 0.08;
	glMaterialfv(GL_FRONT, GL_DIFFUSE, localColor19);
	drawEdge(0.343279, 0, 0.939234, pnt19.x(), pnt19.y(), pnt19.z()); // p1, p6
	localColor19[1] -= 0.08;
	glMaterialfv(GL_FRONT, GL_DIFFUSE, localColor19);
	drawEdge(0.106079, 0.326477, 0.939234, pnt19.x(), pnt19.y(), pnt19.z()); // p2, p3
	localColor19[1] -= 0.08;
	glMaterialfv(GL_FRONT, GL_DIFFUSE, localColor19);
	drawEdge(0.212158, 0.652955, 0.727076, pnt19.x(), pnt19.y(), pnt19.z()); // p3, p4
	localColor19[1] -= 0.08;
	glMaterialfv(GL_FRONT, GL_DIFFUSE, localColor19);
	drawEdge(0.555436, 0.652955, 0.514918, pnt19.x(), pnt19.y(), pnt19.z()); // p4
	localColor19[1] -= 0.08;
	glMaterialfv(GL_FRONT, GL_DIFFUSE, localColor19);
	drawEdge(0.792636, 0.326477, 0.514918, pnt19.x(), pnt19.y(), pnt19.z()); // p5, p4
	localColor19[1] -= 0.08;
	glMaterialfv(GL_FRONT, GL_DIFFUSE, localColor19);
	drawEdge(0.686557, 0, 0.727076, pnt19.x(), pnt19.y(), pnt19.z()); // p6, p5


//Sphere Face #20
	r1 += 0.05;
	g1 = (g1>.08)?0.75:0.9;
	b1 -= 0.05;
	GLfloat localColor20[4]; localColor20[0] = r1; localColor20[1] = g1; localColor20[2] = b1; localColor20[3] = 1;
	glMaterialfv(GL_FRONT, GL_DIFFUSE, localColor20);
	drawEdge( 0.277718, -0.201774, -0.939234, 0.277718, 0.201774, -0.939234);
	drawEdge( 0.277718, -0.201774, -0.939234, 0.555436, -0.403548, -0.727076);
	// drawEdge( 0.277718, 0.201774, -0.939234, 0.555436, 0.403548, -0.727076);
	drawEdge( 0.555436, -0.403548, -0.727076, 0.833155, -0.201774, -0.514918);
	drawEdge( 0.555436, 0.403548, -0.727076, 0.833155, 0.201774, -0.514918);
	//- drawEdge( 0.833155, -0.201774, -0.514918, 0.833155, 0.201774, -0.514918);

	Pt3 pnt20(0.343279, 0, 0.939234);
	Pt3 pnt20a(0.555436, 0.652955, 0.514918);

	pnt20 += pnt20a;
	pnt20 *= 0.5;
	pnt20.Normalize();

	localColor20[1] -= 0.08;
	glMaterialfv(GL_FRONT, GL_DIFFUSE, localColor20);
	drawEdge(0.343279, 0, 0.939234, pnt20.x(), pnt20.y(), pnt20.z()); // p1, p6
	localColor20[1] -= 0.08;
	glMaterialfv(GL_FRONT, GL_DIFFUSE, localColor20);
	drawEdge(0.106079, 0.326477, 0.939234, pnt20.x(), pnt20.y(), pnt20.z()); // p2, p3
	localColor20[1] -= 0.08;
	glMaterialfv(GL_FRONT, GL_DIFFUSE, localColor20);
	drawEdge(0.212158, 0.652955, 0.727076, pnt20.x(), pnt20.y(), pnt20.z()); // p3, p4
	localColor20[1] -= 0.08;
	glMaterialfv(GL_FRONT, GL_DIFFUSE, localColor20);
	drawEdge(0.555436, 0.652955, 0.514918, pnt20.x(), pnt20.y(), pnt20.z()); // p4
	localColor20[1] -= 0.08;
	glMaterialfv(GL_FRONT, GL_DIFFUSE, localColor20);
	drawEdge(0.792636, 0.326477, 0.514918, pnt20.x(), pnt20.y(), pnt20.z()); // p5, p4
	localColor20[1] -= 0.08;
	glMaterialfv(GL_FRONT, GL_DIFFUSE, localColor20);
	drawEdge(0.686557, 0, 0.727076, pnt20.x(), pnt20.y(), pnt20.z()); // p6, p5



	glDisable(GL_LIGHTING);
	glDisable(GL_CULL_FACE);
}

//--------------------------------------------------------------------
void Sphere::calc()
{
	Int nRings = getNumYSteps()-1;
	Int nSegments = getNumXSteps();
	Pt3 c = getCenter();
	Pt3 u = getMajorAxis();

	//Float R = GetThickness();

	Float radius = 1;

	setTop(Pt3(0,0,radius));
	setBottom(Pt3(0,0,-radius));

	vector<vector<Pt3> > &rings = getRings();
	rings.resize(nRings);

	int centerRing = nRings/2;

	for(Int k=0; k<=nRings-1; ++k) {

		rings[k].resize(nSegments);

		Float ringAngle = (((Float)(centerRing-k))/(Float)(nRings+1)) * M_PI;
		Float ringRadius = cos(ringAngle);
		Float ringZ = sin(ringAngle);

		for(Int i=0; i<nSegments; ++i) {
			Float t = (Float)i / (Float)nSegments;
			Float a = t * M_2PI;

			Float ca = cos(a);
			Float sa = sin(a);

			//Pt3 x = right[k] * ca + up[k] * sa;
			//Pt3 y = right[k] * sa - up[k] * ca;

			//Pt3 p = x + y;

			rings[k][i] = Pt3(ringRadius*ca, ringRadius*sa, ringZ);
			double *dp = rings[k][i].Coord();
			TRACE("calc-ring %d, seg %d, %g, %g, %g\n", k, i, dp[0], dp[1], dp[2]);

			//normals[k][i] = p;
		}
	}
}

//--------------------------------------------------------------------
void Sphere::material()
{
	glEnable(GL_LIGHTING);

	//glCullFace(GL_BACK);
	//glEnable(GL_CULL_FACE);

    /*static*/ GLfloat mat_front_ambient_normal[4] = {1, 0, 0, 0.5f};
    //static GLfloat mat_front_diffuse_normal[4] = {0.5, 0.9, 0.5, 0.5};
    /*static*/ GLfloat mat_front_diffuse_normal[4] = {1, 1, 1, 1};

    static GLfloat mat_back_ambient_normal[4] = {0.3f, 0.1f, 0.4f, 0.5f};
    //static GLfloat mat_back_diffuse_normal[4] = {0.9, 0.5, 0.5, 0.5};
    //static GLfloat mat_back_diffuse_normal[4] = {0.3f, 0.1f, 0.1f, 0.5f};

    //static GLfloat mat_glow[4] = {0.0f, 0.0f, 1.0f, 0.5f};
    static GLfloat mat_null[4] = {0.0f, 0.0f, 0.0f, 1};

    //static GLfloat mat_selected[4] = {1.0f, 0.0f, 0.0f, 0.5f};
    //static GLfloat mat_touched[4] = {0.0f, 0.1f, 0.1f, 0.5f};
    static GLfloat mat_specular[4] = {1, 1, 1, 1};

    static GLfloat mat_shininess[1] = {128.0};

    //glMaterialfv(GL_FRONT, GL_AMBIENT, mat_front_ambient_normal);
    glMaterialfv(GL_FRONT, GL_AMBIENT, mat_null);
    glMaterialfv(GL_FRONT, GL_DIFFUSE, mat_front_diffuse_normal);

    glMaterialfv(GL_BACK, GL_AMBIENT, mat_back_ambient_normal);
    //glMaterialfv(GL_BACK, GL_DIFFUSE, mat_back_diffuse_normal);
    glMaterialfv(GL_FRONT,GL_EMISSION, mat_null);

    //glMaterialfv(GL_FRONT_AND_BACK, GL_SPECULAR, mat_specular);
    //glMaterialfv(GL_FRONT_AND_BACK, GL_SHININESS, mat_shininess);

    glMaterialfv(GL_FRONT, GL_SPECULAR, mat_specular);
    glMaterialfv(GL_FRONT, GL_SHININESS, mat_shininess);

	//glShadeModel(GL_SMOOTH);
	//glShadeModel(GL_FLAT);

	//glPolygonMode(GL_FRONT_AND_BACK, GL_LINE);
	//glPolygonMode(GL_FRONT, GL_FILL);
	//glPolygonMode(GL_FRONT_AND_BACK, GL_FILL);
}

//--------------------------------------------------------------------

//--------------------------------------------------------------------
void Sphere::setNumXSteps(Int s)
{
	m_xSteps = s;
}

void Sphere::setNumYSteps(Int s)
{
	m_ySteps = s;
}

//--------------------------------------------------------------------
void Sphere::setCenter(const Pt3 &c)
{
	m_center = c;
}

//--------------------------------------------------------------------
void Sphere::setMajorAxis(const Pt3 &a)
{
	m_majorAxis = a;
}

//--------------------------------------------------------------------
void Sphere::setTop(const Pt3 &a)
{
	m_top = a;
}
//--------------------------------------------------------------------
void Sphere::setBottom(const Pt3 &a)
{
	m_bottom = a;
}
