// OpenGLView.cpp : implementation of the COpenGLView class
//
#include "stdafx.h"
#include "OpenGL.h"

#include <gl\glu.h>
#include <world.h>
#include <basic.h>
#include <pt2.h>

#include "OpenGLDoc.h"
#include "OpenGLView.h"
static bool g_isRotating = true;

#ifdef _DEBUG
#define new DEBUG_NEW
#undef THIS_FILE
static char THIS_FILE[] = __FILE__;
#endif

/////////////////////////////////////////////////////////////////////////////
// COpenGLView

IMPLEMENT_DYNCREATE(COpenGLView, CView)

BEGIN_MESSAGE_MAP(COpenGLView, CView)
	//{{AFX_MSG_MAP(COpenGLView)
	ON_WM_CREATE()
	ON_WM_DESTROY()
	ON_WM_ERASEBKGND()
	ON_WM_SIZE()
	ON_WM_KEYDOWN()
	ON_WM_KEYUP()
	ON_WM_LBUTTONDBLCLK()
	ON_WM_LBUTTONDOWN()
	ON_WM_MBUTTONDOWN()
	ON_WM_LBUTTONUP()
	ON_WM_MOUSEMOVE()
	ON_WM_RBUTTONDBLCLK()
	ON_WM_RBUTTONDOWN()
	ON_WM_RBUTTONUP()
	ON_COMMAND(ID_RENDER_FILL, OnRenderFill)
	ON_COMMAND(ID_RENDER_LINE, OnRenderLine)
	ON_COMMAND(ID_RENDER_POINT, OnRenderPoint)
	ON_UPDATE_COMMAND_UI(ID_RENDER_FILL, OnUpdateRenderFill)
	ON_UPDATE_COMMAND_UI(ID_RENDER_LINE, OnUpdateRenderLine)
	ON_UPDATE_COMMAND_UI(ID_RENDER_POINT, OnUpdateRenderPoint)
	ON_COMMAND(ID_RENDER_SHADING_FLAT, OnRenderShadingFlat)
	ON_UPDATE_COMMAND_UI(ID_RENDER_SHADING_FLAT, OnUpdateRenderShadingFlat)
	ON_COMMAND(ID_RENDER_SHADING_SMOOTH, OnRenderShadingSmooth)
	ON_UPDATE_COMMAND_UI(ID_RENDER_SHADING_SMOOTH, OnUpdateRenderShadingSmooth)
	ON_WM_TIMER()
	ON_UPDATE_COMMAND_UI(ID_ROTATE, OnUpdateRotate)
	ON_COMMAND(ID_ROTATE, OnRotate)
	//}}AFX_MSG_MAP
END_MESSAGE_MAP()

/////////////////////////////////////////////////////////////////////////////
// COpenGLView construction/destruction

COpenGLView::COpenGLView() :
	left_(-1.0),
	right_(1.0),
	top_(1.0),
	bottom_(-1.0),
	near_(-10.0),
	far_(100.0),
	zoom_(1.0),
	xang_(45.0),
	yang_(-115.0),
	eye_(0.0, 0.0, -10.0),
    focus_(0.0, 0.0, 0.0),
    up_(0.0, 1.0, 0.0)
{
	// TODO: add construction code here
	w_ = new World();
}

COpenGLView::~COpenGLView()
{
}

BOOL COpenGLView::PreCreateWindow(CREATESTRUCT& cs)
{
	// TODO: Modify the Window class or styles here by modifying
	//  the CREATESTRUCT cs

	cs.style |= WS_CLIPSIBLINGS | WS_CLIPCHILDREN;

	return CView::PreCreateWindow(cs);
}

/////////////////////////////////////////////////////////////////////////////
// COpenGLView drawing

void COpenGLView::OnDraw(CDC* pDC)
{
	COpenGLDoc* pDoc = GetDocument();
	ASSERT_VALID(pDoc);

	wglMakeCurrent(pDC->m_hDC, m_hRC);
	glViewport(0, 0, GetWidth(), GetHeight());
	glClear( GL_COLOR_BUFFER_BIT | GL_DEPTH_BUFFER_BIT );
	
	//-----------------------------------------------------------------------
	glMatrixMode(GL_PROJECTION);
    glLoadIdentity();

	double z = GetZoom();
    double w = (double)GetWidth();
    double h = (double)GetHeight();
    if(w <= h) {
		SetLeft(-z);
		SetRight(z);
		SetBottom(-z*h/w);
		SetTop(z*h/w);
    } else {
		SetLeft(-z*w/h);
		SetRight(z*w/h);
		SetBottom(-z);
		SetTop(z);
    }
    glOrtho(GetLeft(),GetRight(),GetBottom(),GetTop(),GetNear(),GetFar());
	//-----------------------------------------------------------------------
    glMatrixMode(GL_MODELVIEW);
    glLoadIdentity();


	Pt3 &eye = GetEye();
	Pt3 &focus = GetFocus();
	Pt3 &up = GetUp();

    gluLookAt(
		eye.x(), eye.y(), eye.z(),
		focus.x(), focus.y(), focus.z(),
		up.x(), up.y(), up.z());
  
    glRotated(GetYAng(), 1.0, 0.0, 0.0);
    glRotated(GetXAng(), 0.0, 0.0, 1.0);

	//-----------------------------------------------------------------------
	glPushMatrix();
		// draw some stuff
		glColor3d(1.0,1.0,1.0);
		glLineWidth(3.0);
		glBegin(GL_LINES);
			glVertex3d(0.0,0.0,0.0);
			glVertex3d(1.0,0.0,0.0);
			glVertex3d(0.0,0.0,0.0);
			glVertex3d(0.0,1.0,0.0);
			glVertex3d(0.0,0.0,0.0);
			glVertex3d(0.0,0.0,1.0);
		glEnd();
		glLineWidth(1.0);
	glPopMatrix();

	//-----------------------------------------------------------------------
	glPushMatrix();
		World *world = GetWorld();
		if(world) {
			world->InitLight();
			world->Draw();
		}
	glPopMatrix();

	//-----------------------------------------------------------------------
	glFinish();
	SwapBuffers( pDC->m_hDC );
	wglMakeCurrent(pDC->m_hDC, NULL);

	// TODO: add draw code for native data here
}

/////////////////////////////////////////////////////////////////////////////
// COpenGLView diagnostics

#ifdef _DEBUG
void COpenGLView::AssertValid() const
{
	CView::AssertValid();
}

void COpenGLView::Dump(CDumpContext& dc) const
{
	CView::Dump(dc);
}

COpenGLDoc* COpenGLView::GetDocument() // non-debug version is inline
{
	ASSERT(m_pDocument->IsKindOf(RUNTIME_CLASS(COpenGLDoc)));
	return (COpenGLDoc*)m_pDocument;
}
#endif //_DEBUG

/////////////////////////////////////////////////////////////////////////////
// COpenGLView message handlers
BOOL LoadBMP(TCHAR* szFileName);
int COpenGLView::OnCreate(LPCREATESTRUCT lpCreateStruct) 
{
	if (CView::OnCreate(lpCreateStruct) == -1)
		return -1;
	
	// TODO: Add your specialized creation code here
	static PIXELFORMATDESCRIPTOR pfd = {
        sizeof(PIXELFORMATDESCRIPTOR),  // size of this pfd
        1,                              // version number
        PFD_DRAW_TO_WINDOW |            // support window
          PFD_SUPPORT_OPENGL |          // support OpenGL
          PFD_DOUBLEBUFFER,             // double buffered
        PFD_TYPE_RGBA,                  // RGBA type
        24, //16,                             // 24-bit color depth
        0, 0, 0, 0, 0, 0,               // color bits ignored
        0,                              // no alpha buffer
        0,                              // shift bit ignored
        0,                              // no accumulation buffer
        0, 0, 0, 0,                     // accum bits ignored
        32,//16,                             // 16-bit z-buffer
        0,                              // no stencil buffer
        0,                              // no auxiliary buffer
        PFD_MAIN_PLANE,                 // main layer
        0,                              // reserved
        0, 0, 0                         // layer masks ignored
    };

	m_pDC = new CClientDC(this);
    //m_PixelFormat = ChoosePixelFormat(m_pDC->GetSafeHdc(), &pfd);
    //SetPixelFormat(m_pDC->GetSafeHdc(), m_PixelFormat, &pfd);

	//CClientDC clientDC(this);
	int pixelFormat = ChoosePixelFormat(m_pDC->m_hDC, &pfd);
	BOOL success = SetPixelFormat(m_pDC->m_hDC, pixelFormat, &pfd);
	//TRACE("set pixel format success == %d\n",success);
	m_hRC = wglCreateContext(m_pDC->m_hDC);

	glClearColor(0.0, 0.0, 0.0, 1.0);
	glClearDepth(1.0);
	glEnable(GL_DEPTH_TEST);
	BOOL result = LoadBMP("Zork.bmp");

    SetTimer(1, 200, NULL);
	return 0;
}

//--------------------------------------------------------------------
void COpenGLView::OnDestroy() 
{
	CView::OnDestroy();
	
	// TODO: Add your message handler code here

	wglDeleteContext( m_hRC );
	
}

//--------------------------------------------------------------------
BOOL COpenGLView::OnEraseBkgnd(CDC* pDC) 
{
	// TODO: Add your message handler code here and/or call default
	
	//return CView::OnEraseBkgnd(pDC);
	return TRUE;
}

//--------------------------------------------------------------------
void COpenGLView::OnSize(UINT nType, int cx, int cy) 
{
	TRACE("OnSize x = %d    y = %d\n",cx,cy);

	CView::OnSize(nType, cx, cy);
	
	// TODO: Add your message handler code here
	SetWidth(cx);
	SetHeight(cy);

	//glViewport(0, 0, GetWidth(), GetHeight());
}

//--------------------------------------------------------------------
void COpenGLView::OnKeyDown(UINT nChar, UINT nRepCnt, UINT nFlags) 
{
	// TODO: Add your message handler code here and/or call default
	
	CView::OnKeyDown(nChar, nRepCnt, nFlags);
}

//--------------------------------------------------------------------
void COpenGLView::OnKeyUp(UINT nChar, UINT nRepCnt, UINT nFlags) 
{
	// TODO: Add your message handler code here and/or call default
	
	CView::OnKeyUp(nChar, nRepCnt, nFlags);
}

//--------------------------------------------------------------------
void COpenGLView::OnLButtonDblClk(UINT nFlags, CPoint point) 
{
	// TODO: Add your message handler code here and/or call default
	
	CView::OnLButtonDblClk(nFlags, point);
}

//--------------------------------------------------------------------
void COpenGLView::OnLButtonDown(UINT nFlags, CPoint point) 
{
	// TODO: Add your message handler code here and/or call default
   GenericButtonDown(nFlags, point);
   return;
	
	CView::OnLButtonDown(nFlags, point);
}

//--------------------------------------------------------------------
void COpenGLView::OnLButtonUp(UINT nFlags, CPoint point) 
{
	// TODO: Add your message handler code here and/or call default
	GenericButtonUp(nFlags, point);
   return;

	CView::OnLButtonUp(nFlags, point);
}

//--------------------------------------------------------------------
void COpenGLView::OnMButtonDown(UINT nFlags, CPoint point) 
{
	// TODO: Add your message handler code here and/or call default
   GenericButtonDown(nFlags, point);
   return;
	
	CView::OnMButtonDown(nFlags, point);
}

//--------------------------------------------------------------------
void COpenGLView::OnMButtonUp(UINT nFlags, CPoint point) 
{
	// TODO: Add your message handler code here and/or call default
	GenericButtonUp(nFlags, point);
   return;

	CView::OnMButtonUp(nFlags, point);
}

//--------------------------------------------------------------------
void COpenGLView::OnMouseMove(UINT nFlags, CPoint point) 
{
	// TODO: Add your message handler code here and/or call default
	
	CView::OnMouseMove(nFlags, point);

   //------------------------------------------------------
	// MK_CONTROL   Set if the CTRL key is down.
	// MK_LBUTTON   Set if the left mouse button is down.
	// MK_MBUTTON   Set if the middle mouse button is down.
	// MK_RBUTTON   Set if the right mouse button is down.
	// MK_SHIFT   Set if the SHIFT key is down
   //------------------------------------------------------

	BOOL redraw = FALSE;
	BOOL setpt = FALSE;

   BOOL rotate_view = FALSE;
   BOOL pan_view = FALSE;
   BOOL zoom_view = FALSE;

   //-------------------------------------
   // Let's decide which method to invoke.
   //-------------------------------------
   if((nFlags & MK_MBUTTON) ||
      (nFlags & MK_LBUTTON) && (nFlags & MK_SHIFT))
   {
TRACE("pan_view\n");
      pan_view = TRUE;
   }
   else if((nFlags & MK_RBUTTON) ||
      (nFlags & MK_LBUTTON) && (nFlags & MK_CONTROL))
   {
TRACE("zoom_view\n");
      zoom_view = TRUE;
   }
   else if(nFlags & MK_LBUTTON)
   {
TRACE("rotate_view\n");
      rotate_view = TRUE;
   }

   //---------------------------
   // Finally act on the method.
   //---------------------------
	if(rotate_view) 
	{
		double x = GetXAng();
		double y = GetYAng();

		//---------------------------------------------------------
		Float w = (Float)GetWidth();
		Float h = (Float)GetHeight();
		Float scaled_xi = lin_scale(0.0, w, 0.0, 180.0, GetMX());
		Float scaled_xf = lin_scale(0.0, w, 0.0, 180.0, point.x);
		Float scaled_yi = lin_scale(0.0, h, 0.0, 180.0, GetMY());
		Float scaled_yf = lin_scale(0.0, h, 0.0, 180.0, point.y);
		Float dx = scaled_xf - scaled_xi;
		Float dy = scaled_yf - scaled_yi;
		//---------------------------------------------------------

		SetXAng(x + dx);
		SetYAng(y - dy);

		setpt = TRUE;
		redraw = TRUE;
	}
	else if(zoom_view)
	{
		int my = GetMY();

		Float h = (Float)(GetHeight());
		Float delta_y = (Float)(my - point.y); // upside down screen!

		Float zoom_factor = lin_scale(0.0, h, 1.0, 3.0, fabs(delta_y));
		if(delta_y < 0.0)
		{
			zoom_factor = 1.0 / zoom_factor;
		}
		Float zoom = GetZoom();
		zoom = zoom * zoom_factor;
		SetZoom(zoom);

		setpt = TRUE;
		redraw = TRUE;
	}
	else if(pan_view)
	{
		//---------------------------------------------------------
		Float w = (Float)(GetWidth());
		Float h = (Float)(GetHeight());
		Float scaled_xi = lin_scale(0.0, w, GetLeft(), GetRight(), GetMX());
		Float scaled_xf = lin_scale(0.0, w, GetLeft(), GetRight(), point.x);
		Float scaled_yi = lin_scale(0.0, h, GetBottom(), GetTop(), GetMY());
		Float scaled_yf = lin_scale(0.0, h, GetBottom(), GetTop(), point.y);
		Float dx = scaled_xf - scaled_xi;
		Float dy = scaled_yf - scaled_yi;
		//---------------------------------------------------------

		PanView(Pt2(dx,dy));

		setpt = TRUE;
		redraw = TRUE;
	}

   //-------------------------------
   // Perform any cleanup necessary.
   //-------------------------------
	if(setpt)
	{
		SetMX(point.x);
		SetMY(point.y);
	}

	if(redraw)
	{
		Invalidate(FALSE);
	}
}

//--------------------------------------------------------------------
void COpenGLView::PanView(const Pt2 &p)
{
	Pt3 &eye = GetEye();
	Pt3 &focus = GetFocus();
	Pt3 &up = GetUp();

	Pt3 forward = focus - eye;
	forward.Normalize();

	Pt3 right = CrossProduct(forward, up);

	Pt3 delta = p.y() * up - p.x() * right;

	SetEye(eye + delta);
	SetFocus(focus + delta);
}

//--------------------------------------------------------------------
void COpenGLView::OnRButtonDblClk(UINT nFlags, CPoint point) 
{
	// TODO: Add your message handler code here and/or call default
	
	CView::OnRButtonDblClk(nFlags, point);
}

//--------------------------------------------------------------------
void COpenGLView::OnRButtonDown(UINT nFlags, CPoint point) 
{
	// TODO: Add your message handler code here and/or call default
   GenericButtonDown(nFlags, point);
   return;
	
	CView::OnRButtonDown(nFlags, point);
}

//--------------------------------------------------------------------
void COpenGLView::OnRButtonUp(UINT nFlags, CPoint point) 
{
	// TODO: Add your message handler code here and/or call default
	GenericButtonUp(nFlags, point);
   return;

	CView::OnRButtonUp(nFlags, point);
}

//--------------------------------------------------------------------
void COpenGLView::SetWidth(int w)
{
	width_ = w;
}

//--------------------------------------------------------------------
void COpenGLView::SetHeight(int h)
{
	height_ = h;
}

//--------------------------------------------------------------------
void COpenGLView::SetLeft(double l)
{
	left_ = l;
}

//--------------------------------------------------------------------
void COpenGLView::SetRight(double r)
{
	right_ = r;
}

//--------------------------------------------------------------------
void COpenGLView::SetTop(double t)
{
	top_ = t;
}

//--------------------------------------------------------------------
void COpenGLView::SetBottom(double b)
{
	bottom_ = b;
}

//--------------------------------------------------------------------
void COpenGLView::SetNear(double n)
{
	near_ = n;
}

//--------------------------------------------------------------------
void COpenGLView::SetFar(double f)
{
	far_ = f;
}

//--------------------------------------------------------------------
void COpenGLView::SetZoom(double z)
{
	zoom_ = z;
}

//--------------------------------------------------------------------
void COpenGLView::SetXAng(double a)
{
	xang_ = a;
}

//--------------------------------------------------------------------
void COpenGLView::SetYAng(double a)
{
	yang_ = a;
}

//--------------------------------------------------------------------
void COpenGLView::SetMX(int x)
{
	mx_ = x;
}

//--------------------------------------------------------------------
void COpenGLView::SetMY(int y)
{
	my_ = y;
}

//--------------------------------------------------------------------
void COpenGLView::OnRenderFill() 
{
	World *w = GetWorld();
	if(w) {
		w->SetPolygonMode(GL_FILL);
		Invalidate(FALSE);
	}
}

//--------------------------------------------------------------------
void COpenGLView::OnRenderLine() 
{
	World *w = GetWorld();
	if(w) {
		w->SetPolygonMode(GL_LINE);
		Invalidate(FALSE);
	}
}

//--------------------------------------------------------------------
void COpenGLView::OnRenderPoint() 
{
	World *w = GetWorld();
	if(w) {
		w->SetPolygonMode(GL_POINT);
		Invalidate(FALSE);
	}	
}

//--------------------------------------------------------------------
void COpenGLView::OnUpdateRenderFill(CCmdUI* pCmdUI) 
{
	if(!pCmdUI) return;
	World *w = GetWorld();
	if(w) {
		if(GL_FILL == w->GetPolygonMode()) {
			pCmdUI->SetCheck(1);
		} else {
			pCmdUI->SetCheck(0);
		}
	}	
}

//--------------------------------------------------------------------
void COpenGLView::OnUpdateRenderLine(CCmdUI* pCmdUI) 
{
	if(!pCmdUI) return;
	World *w = GetWorld();
	if(w) {
		if(GL_LINE == w->GetPolygonMode()) {
			pCmdUI->SetCheck(1);
		} else {
			pCmdUI->SetCheck(0);
		}
	}	
}

//--------------------------------------------------------------------
void COpenGLView::OnUpdateRenderPoint(CCmdUI* pCmdUI) 
{
	if(!pCmdUI) return;
	World *w = GetWorld();
	if(w) {
		if(GL_POINT == w->GetPolygonMode()) {
			pCmdUI->SetCheck(1);
		} else {
			pCmdUI->SetCheck(0);
		}
	}	
}

//--------------------------------------------------------------------
void COpenGLView::OnRenderShadingFlat() 
{
	World *w = GetWorld();
	if(w) {
		w->SetShadeModel(GL_FLAT);
		Invalidate(FALSE);
	}
}

//--------------------------------------------------------------------
void COpenGLView::OnUpdateRenderShadingFlat(CCmdUI* pCmdUI) 
{
	if(!pCmdUI) return;
	World *w = GetWorld();
	if(w) {
		if(GL_FLAT == w->GetShadeModel()) {
			pCmdUI->SetCheck(1);
		} else {
			pCmdUI->SetCheck(0);
		}
	}	
}

//--------------------------------------------------------------------
void COpenGLView::OnRenderShadingSmooth() 
{
	World *w = GetWorld();
	if(w) {
		w->SetShadeModel(GL_SMOOTH);
		Invalidate(FALSE);
	}
}

//--------------------------------------------------------------------
void COpenGLView::OnUpdateRenderShadingSmooth(CCmdUI* pCmdUI) 
{
	if(!pCmdUI) return;
	World *w = GetWorld();
	if(w) {
		if(GL_SMOOTH == w->GetShadeModel()) {
			pCmdUI->SetCheck(1);
		} else {
			pCmdUI->SetCheck(0);
		}
	}	
}

//--------------------------------------------------------------------
void COpenGLView::SetEye(const Pt3 &e)
{
	eye_ = e;
}

//--------------------------------------------------------------------
void COpenGLView::SetFocus(const Pt3 &f)
{
	focus_ = f;
}

//--------------------------------------------------------------------
void COpenGLView::SetUp(const Pt3 &u)
{
	up_ = u;
}

//--------------------------------------------------------------------
void COpenGLView::GenericButtonDown(UINT /* nFlags */, CPoint point)
{
	//-----------------
	// reset the values
	//-----------------
	SetMX(point.x);
	SetMY(point.y);

   SetCapture();
}

//--------------------------------------------------------------------
void COpenGLView::GenericButtonUp(UINT /* nFlags */, CPoint /* point */)
{
   ReleaseCapture();
}

//--------------------------------------------------------------------
double g_bulge = 1;
static double increment = 0.1;
void COpenGLView::OnTimer(UINT nIDEvent)
{
	if (!g_isRotating)
		return;

	double angle = GetXAng();
	SetXAng(angle+1);
	Invalidate(FALSE);
}

void COpenGLView::OnUpdateRotate(CCmdUI* pCmdUI) 
{
	pCmdUI->SetCheck(g_isRotating);
}

void COpenGLView::OnRotate() 
{
	g_isRotating = !g_isRotating;	
}
