//--------------------------------------------------------------------
#include "stdafx.h"
#include <sphere.h>
#include <gl\gl.h>

//--------------------------------------------------------------------
Sphere::Sphere()
{
	setMajorAxis(Pt3(1.0, 0.0, 0.0));

	setNumXSteps(8);
	setNumYSteps(4);

	calc();
}

//--------------------------------------------------------------------
Sphere::~Sphere()
{

}

//--------------------------------------------------------------------
void Sphere::Draw()
{
	//glColor3d(0.0, 1.0, 0.0);

	material();

	vector<vector<Pt3> > &rings = getRings();

	Int r = rings.size();
	for(Int i=0; i<r-1; ++i) {
		glBegin(GL_QUAD_STRIP);
			Int ri = rings[i].size();
			for(Int j=0; j<ri; ++j) {

				glNormal3dv(rings[i][j].Coord());
				glVertex3dv(rings[i][j].Coord());

				glNormal3dv(rings[i+1][j].Coord());
				glVertex3dv(rings[i+1][j].Coord());
			}

			glNormal3dv(rings[i][0].Coord());
			glVertex3dv(rings[i][0].Coord());

			glNormal3dv(rings[i+1][0].Coord());
			glVertex3dv(rings[i+1][0].Coord());

		glEnd();
	}

	glBegin(GL_TRIANGLE_FAN);

		glNormal3dv(getTop().Coord());
		glVertex3dv(getTop().Coord());

		Int ri = rings[0].size();
		for(Int j=0; j<ri; ++j) {
			glNormal3dv(rings[0][j].Coord());
			glVertex3dv(rings[0][j].Coord());
		}

		glNormal3dv(rings[0][0].Coord());
		glVertex3dv(rings[0][0].Coord());

	glEnd();
	
	glBegin(GL_TRIANGLE_FAN);

		glNormal3dv(getBottom().Coord());
		glVertex3dv(getBottom().Coord());

		glNormal3dv(rings[r-1][0].Coord());
		glVertex3dv(rings[r-1][0].Coord());

		ri = rings[r-1].size();
		for(j=ri-1; j>=0; --j) {
			glNormal3dv(rings[r-1][j].Coord());
			glVertex3dv(rings[r-1][j].Coord());
		}


	glEnd();
	

	glDisable(GL_LIGHTING);
	glDisable(GL_CULL_FACE);
}

//--------------------------------------------------------------------
void Sphere::calc()
{
	Int nRings = getNumYSteps()-1;
	Int nSegments = getNumXSteps();
	Pt3 c = getCenter();
	Pt3 u = getMajorAxis();

	//Float R = GetThickness();

	Float radius = 1;

	setTop(Pt3(0,0,radius));
	setBottom(Pt3(0,0,-radius));

	vector<vector<Pt3> > &rings = getRings();
	rings.resize(nRings);

	int centerRing = nRings/2;

	for(Int k=0; k<=nRings-1; ++k) {

		rings[k].resize(nSegments);

		Float ringAngle = (((Float)(centerRing-k))/(Float)(nRings+1)) * M_PI;
		Float ringRadius = cos(ringAngle);
		Float ringZ = sin(ringAngle);

		for(Int i=0; i<nSegments; ++i) {
			Float t = (Float)i / (Float)nSegments;
			Float a = t * M_2PI;

			Float ca = cos(a);
			Float sa = sin(a);

			//Pt3 x = right[k] * ca + up[k] * sa;
			//Pt3 y = right[k] * sa - up[k] * ca;

			//Pt3 p = x + y;

			rings[k][i] = Pt3(ringRadius*ca, ringRadius*sa, ringZ);
			double *dp = rings[k][i].Coord();
			TRACE("calc-ring %d, seg %d, %g, %g, %g\n", k, i, dp[0], dp[1], dp[2]);

			//normals[k][i] = p;
		}
	}
}

//--------------------------------------------------------------------
void Sphere::material()
{
	glEnable(GL_LIGHTING);

	glCullFace(GL_BACK);
	glEnable(GL_CULL_FACE);

    /*static*/ GLfloat mat_front_ambient_normal[4] = {1, 0, 0, 0.5f};
    //static GLfloat mat_front_diffuse_normal[4] = {0.5, 0.9, 0.5, 0.5};
    /*static*/ GLfloat mat_front_diffuse_normal[4] = {1, 0, 0, 1};

    //static GLfloat mat_back_ambient_normal[4] = {0.9f, 0.1f, 0.1f, 0.5f};
    //static GLfloat mat_back_diffuse_normal[4] = {0.9, 0.5, 0.5, 0.5};
    //static GLfloat mat_back_diffuse_normal[4] = {0.3f, 0.1f, 0.1f, 0.5f};

    //static GLfloat mat_glow[4] = {0.0f, 0.0f, 1.0f, 0.5f};
    static GLfloat mat_null[4] = {0.0f, 0.0f, 0.0f, 0.5f};

    //static GLfloat mat_selected[4] = {1.0f, 0.0f, 0.0f, 0.5f};
    //static GLfloat mat_touched[4] = {0.0f, 0.1f, 0.1f, 0.5f};
    static GLfloat mat_specular[4] = {0.1f, 0.1f, 0.1f, 0.5f};

    static GLfloat mat_shininess[1] = {128.0};

    //glMaterialfv(GL_FRONT, GL_AMBIENT, mat_front_ambient_normal);
    glMaterialfv(GL_FRONT, GL_AMBIENT, mat_null);
    glMaterialfv(GL_FRONT, GL_DIFFUSE, mat_front_diffuse_normal);

    //glMaterialfv(GL_BACK, GL_AMBIENT, mat_back_ambient_normal);
    //glMaterialfv(GL_BACK, GL_DIFFUSE, mat_back_diffuse_normal);
    glMaterialfv(GL_FRONT,GL_EMISSION, mat_null);

    //glMaterialfv(GL_FRONT_AND_BACK, GL_SPECULAR, mat_specular);
    //glMaterialfv(GL_FRONT_AND_BACK, GL_SHININESS, mat_shininess);

    glMaterialfv(GL_FRONT, GL_SPECULAR, mat_specular);
    glMaterialfv(GL_FRONT, GL_SHININESS, mat_shininess);

	//glShadeModel(GL_SMOOTH);
	//glShadeModel(GL_FLAT);

	//glPolygonMode(GL_FRONT_AND_BACK, GL_LINE);
	//glPolygonMode(GL_FRONT, GL_FILL);
	//glPolygonMode(GL_FRONT_AND_BACK, GL_FILL);
}

//--------------------------------------------------------------------

//--------------------------------------------------------------------
void Sphere::setNumXSteps(Int s)
{
	m_xSteps = s;
}

void Sphere::setNumYSteps(Int s)
{
	m_ySteps = s;
}

//--------------------------------------------------------------------
void Sphere::setCenter(const Pt3 &c)
{
	m_center = c;
}

//--------------------------------------------------------------------
void Sphere::setMajorAxis(const Pt3 &a)
{
	m_majorAxis = a;
}

//--------------------------------------------------------------------
void Sphere::setTop(const Pt3 &a)
{
	m_top = a;
}
//--------------------------------------------------------------------
void Sphere::setBottom(const Pt3 &a)
{
	m_bottom = a;
}
