//--------------------------------------------------------------------
#include <borromean.h>
#include <world.h>
#include <gl\gl.h>

//--------------------------------------------------------------------
World::World() : 
	borr_(new Borromean()),
	pgonmode_(GL_FILL),
	shademodel_(GL_SMOOTH)
{
	vector<Pt3> &l = GetLights();
	l.push_back(Pt3( 5.0,  5.0,  5.0));
	l.push_back(Pt3( 5.0, -5.0,  -15.0));
	l.push_back(Pt3(10.0, -10.0, 10.0));
}

//--------------------------------------------------------------------
World::~World()
{
	delete(borr_);
}

//--------------------------------------------------------------------
void World::Draw()
{
	glEnable(GL_DEPTH_TEST);
	Borromean *b = GetBorromean();
	if(b) {
		glPolygonMode(GL_FRONT, GetPolygonMode());
		glShadeModel(GetShadeModel());
		b->Draw();
	}
}

//--------------------------------------------------------------------
void World::InitLight()
{
    static float ambient[] = {0.1f, 0.1f, 0.1f, 1.0f};
    static float diffuse[] = {1.0f, 1.0f, 1.0f, 1.0f};

	vector<Pt3> &l = GetLights();

    for(Int i=0; i<l.size(); ++i) {
        glLightfv(GL_LIGHT0+i, GL_AMBIENT, ambient);
        glLightfv(GL_LIGHT0+i, GL_DIFFUSE, diffuse);
        glLightfv(GL_LIGHT0+i, GL_SPECULAR, diffuse);

        //Light *l = lights[i];
        //Pt3 p = l->Pos();
        //l->SetDir(-p); // good guess?
		Pt3 p = l[i];
        float pos[4] = {(Float)p.x(),  (Float)p.y(),  (Float)p.z(),  1.0};
        //float dir[4] = {(Float)-p.x(), (Float)-p.y(), (Float)-p.z(), 1.0};

        glLightfv(GL_LIGHT0+i, GL_POSITION, pos);
        //glLightdv(GL_LIGHT0+i, GL_SPOT_DIRECTION, dir);
        glEnable(GL_LIGHT0+i);
    }


    static float lmodel_ambient[] = {0.2f, 0.2f, 0.2f, 1.0f};
    glLightModelfv(GL_LIGHT_MODEL_AMBIENT, lmodel_ambient);
    glLightModeli(GL_LIGHT_MODEL_LOCAL_VIEWER, GL_TRUE);
    glLightModeli(GL_LIGHT_MODEL_TWO_SIDE, GL_TRUE);
}

//--------------------------------------------------------------------
void World::SetPolygonMode(GLenum mode)
{
	pgonmode_ = mode;
}

//--------------------------------------------------------------------
void World::SetShadeModel(GLenum mode)
{
	shademodel_ = mode;
}

//--------------------------------------------------------------------